"""Provide basic utilities for pyRevit scripts.

Examples:
    ```python
    from pyrevit import script
    script.clipboard_copy('some text')
    data = script.journal_read('data-key')
    script.exit()
    ```
"""
#pylint: disable=consider-using-f-string

import sys
import os
import os.path as op
import warnings
import re
import codecs
import json
import csv
import pickle

from pyrevit import EXEC_PARAMS, PyRevitException
from pyrevit import coreutils
from pyrevit.coreutils import logger
from pyrevit.coreutils import appdata
from pyrevit.coreutils import envvars
from pyrevit import framework
from pyrevit import revit
from pyrevit import output
from pyrevit import versionmgr
from pyrevit.labs import PyRevit
from pyrevit.revit import ui
import pyrevit.extensions as exts
from pyrevit.extensions.parser import parse_comp_dir
from pyrevit.extensions.genericcomps import GenericUICommand

# suppress any warning generated by native or third-party modules
warnings.filterwarnings("ignore")

#pylint: disable=W0703,C0302,C0103,W0614
mlogger = logger.get_logger(__name__)


DATAFEXT = 'pym'

ICON_SMALL = 16
ICON_MEDIUM = 24
ICON_LARGE = 32

def get_info():
    """Return info on current pyRevit command.

    Returns:
        (pyrevit.extensions.genericcomps.GenericUICommand):
            Command info object
    """
    return get_command_from_path(EXEC_PARAMS.command_path)


def get_command_from_path(comp_path):
    """Returns a pyRevit command object from the given bundle directory.

    Args:
        comp_path (str): Full directory address of the command bundle

    Returns:
        (genericcomps.GenericUICommand): A subclass of pyRevit command object.
    """
    cmds = parse_comp_dir(comp_path, GenericUICommand)
    if cmds:
        return cmds[0]

    return None


def get_script_path():
    """Return script path of the current pyRevit command.

    Returns:
        (str): script path
    """
    return EXEC_PARAMS.command_path


def get_alt_script_path():
    """Return config script path of the current pyRevit command.

    Returns:
        (str): config script path
    """
    return EXEC_PARAMS.command_config_path


def get_bundle_name():
    """Return bundle name of the current pyRevit command.

    Returns:
        (str): bundle name (e.g. MyButton.pushbutton)
    """
    return EXEC_PARAMS.command_bundle


def get_extension_name():
    """Return extension name of the current pyRevit command.

    Returns:
        (str): extension name (e.g. MyExtension.extension)
    """
    return EXEC_PARAMS.command_extension


def get_unique_id():
    """Return unique id of the current pyRevit command.

    Returns:
        (str): command unique id
    """
    return EXEC_PARAMS.command_uniqueid


def get_results():
    """Return command results dictionary for logging.

    Returns:
        (pyrevit.telemetry.record.CommandCustomResults):
            Command results dict
    """
    from pyrevit.telemetry.record import CommandCustomResults
    return CommandCustomResults()


def get_pyrevit_version():
    """Return pyRevit version.

    Returns:
        (pyrevit.versionmgr._PyRevitVersion): pyRevit version provider
    """
    return versionmgr.get_pyrevit_version()


def get_logger():
    """Create and return logger named for current script.

    Returns:
        (pyrevit.coreutils.logger.LoggerWrapper): Logger object
    """
    return logger.get_logger(EXEC_PARAMS.command_name)


def get_output():
    """Return object wrapping output window for current script.

    Returns:
        (pyrevit.output.PyRevitOutputWindow): Output wrapper object
    """
    return output.get_output()


def get_config(section=None):
    """Create and return config section parser object for current script.

    Args:
        section (str, optional): config section name. If not provided,
            it will default to the command name plus the 'config' suffix.

    Returns:
        (pyrevit.coreutils.configparser.PyRevitConfigSectionParser):
            Config section parser object
    """
    from pyrevit.userconfig import user_config
    if not section:
        script_cfg_postfix = '_config'
        section = EXEC_PARAMS.command_name + script_cfg_postfix

    try:
        return user_config.get_section(section)
    except AttributeError:
        return user_config.add_section(section)


def save_config():
    """Save pyRevit config.

    Scripts should call this to save any changes they have done to their
    config section object received from script.get_config() method.
    """
    from pyrevit.userconfig import user_config
    user_config.save_changes()


def reset_config(section=None):
    """Reset pyRevit config.

    Script should call this to reset any save configuration by removing
    section related to current script.

    Args:
        section (str, optional): config section name
    """
    from pyrevit.userconfig import user_config
    if not section:
        script_cfg_postfix = 'config'
        section = EXEC_PARAMS.command_name + script_cfg_postfix
    elif section in [PyRevit.PyRevitConsts.ConfigsCoreSection]:
        raise PyRevitException('Can not remove internal config section: {}'
                               .format(section))

    try:
        user_config.remove_section(section)
        user_config.save_changes()
    except Exception:
        mlogger.debug('Failed resetting config for %s (%s)',
                      EXEC_PARAMS.command_name, section)


def get_universal_data_file(file_id, file_ext, add_cmd_name=False):
    """Return filename to be used by a user script to store data.

    File name is generated in this format:
    ``pyRevit_{file_id}.{file_ext}``

    Examples:
        ```python
        script.get_universal_data_file('mydata', 'data')
        ```
        '/pyRevit_mydata.data'
        ```python
        script.get_universal_data_file('mydata', 'data', add_cmd_name=True)
        ```
        '/pyRevit_Command Name_mydata.data'

    Universal data files are not cleaned up at pyRevit startup.
    Script should manage cleaning up these files.

    Args:
        file_id (str): unique id for the filename
        file_ext (str): file extension
        add_cmd_name (bool, optional): add command name to file name

    Returns:
        (str): full file path
    """
    if add_cmd_name:
        script_file_id = '{}_{}'.format(EXEC_PARAMS.command_name, file_id)
    else:
        script_file_id = file_id

    return appdata.get_universal_data_file(script_file_id, file_ext)


def get_data_file(file_id, file_ext, add_cmd_name=False):
    """Return filename to be used by a user script to store data.

    File name is generated in this format:
    ``pyRevit_{Revit Version}_{file_id}.{file_ext}``

    Examples:
        ```python
        script.get_data_file('mydata', 'data')
        ```
        '/pyRevit_2018_mydata.data'
        ```python
        script.get_data_file('mydata', 'data', add_cmd_name=True)
        ```
        '/pyRevit_2018_Command Name_mydata.data'


    Data files are not cleaned up at pyRevit startup.
    Script should manage cleaning up these files.

    Args:
        file_id (str): unique id for the filename
        file_ext (str): file extension
        add_cmd_name (bool, optional): add command name to file name

    Returns:
        (str): full file path
    """
    if add_cmd_name:
        script_file_id = '{}_{}'.format(EXEC_PARAMS.command_name, file_id)
    else:
        script_file_id = file_id

    return appdata.get_data_file(script_file_id, file_ext)


def get_instance_data_file(file_id, add_cmd_name=False):
    """Return filename to be used by a user script to store data.

    File name is generated in this format:
    ``pyRevit_{Revit Version}_{Process Id}_{file_id}.{file_ext}``

    Examples:
        ```python
        script.get_instance_data_file('mydata')
        ```
        '/pyRevit_2018_6684_mydata.tmp'
        ```python
        script.get_instance_data_file('mydata', add_cmd_name=True)
        ```
        '/pyRevit_2018_6684_Command Name_mydata.tmp'

    Instance data files are cleaned up at pyRevit startup.

    Args:
        file_id (str): unique id for the filename
        add_cmd_name (bool, optional): add command name to file name

    Returns:
        (str): full file path
    """
    if add_cmd_name:
        script_file_id = '{}_{}'.format(EXEC_PARAMS.command_name, file_id)
    else:
        script_file_id = file_id

    return appdata.get_instance_data_file(script_file_id)


def get_document_data_file(file_id, file_ext, add_cmd_name=False):
    """Return filename to be used by a user script to store data.

    File name is generated in this format:
    ``pyRevit_{Revit Version}_{file_id}_{Project Name}.{file_ext}``

    Examples:
        ```python
        script.get_document_data_file('mydata', 'data')
        ```
        '/pyRevit_2018_mydata_Project1.data'
        ```python
        script.get_document_data_file('mydata', 'data', add_cmd_name=True)
        ```
        '/pyRevit_2018_Command Name_mydata_Project1.data'

    Document data files are not cleaned up at pyRevit startup.
    Script should manage cleaning up these files.

    Args:
        file_id (str): unique id for the filename
        file_ext (str): file extension
        add_cmd_name (bool, optional): add command name to file name

    Returns:
        (str): full file path
    """
    proj_info = revit.query.get_project_info()

    if add_cmd_name:
        script_file_id = '{}_{}_{}'.format(EXEC_PARAMS.command_name,
                                           file_id,
                                           proj_info.filename
                                           or proj_info.name)
    else:
        script_file_id = '{}_{}'.format(file_id,
                                        proj_info.filename
                                        or proj_info.name)

    return appdata.get_data_file(script_file_id, file_ext)


def remove_data_file(filepath):
    """Remove given data file."""
    appdata.garbage_data_file(filepath)


def get_bundle_file(file_name):
    """Return full path to file under current script bundle.

    Args:
        file_name (str): bundle file name

    Returns:
        (str): full bundle file path
    """
    return op.join(EXEC_PARAMS.command_path, file_name)


def get_bundle_files(sub_path=None):
    """Return full path to all file under current script bundle.

    Returns:
        (list[str]): list of bundle file paths
    """
    if sub_path:
        command_path = op.join(EXEC_PARAMS.command_path, sub_path)
    else:
        command_path = EXEC_PARAMS.command_path
    return [op.join(command_path, x) for x in os.listdir(command_path)]


def journal_write(data_key, msg):
    """Write key and value to active Revit journal for current command.

    Args:
        data_key (str): data key
        msg (str): data value string
    """
    # Get the StringStringMap class which can write data into.
    data_map = EXEC_PARAMS.command_data.JournalData
    data_map.Clear()

    # Begin to add the support data
    data_map.Add(data_key, msg)


def journal_read(data_key):
    """Read value for provided key from active Revit journal.

    Args:
        data_key (str): data key

    Returns:
        (str): data value string
    """
    # Get the StringStringMap class which can write data into.
    data_map = EXEC_PARAMS.command_data.JournalData

    # Begin to get the support data
    return data_map[data_key]


def get_button():
    """Find and return current script ui button.

    Returns:
        (pyrevit.coreutils.ribbon._PyRevitRibbonButton): ui button object
    """
    from pyrevit.coreutils.ribbon import get_current_ui
    pyrvt_tabs = get_current_ui().get_pyrevit_tabs()
    for tab in pyrvt_tabs:
        button = tab.find_child(EXEC_PARAMS.command_name)
        if button:
            return button
    return None


def get_all_buttons():
    """Find and return all ui buttons matching current script command name.

    Sometimes tools are duplicated across extensions for user access control
    so this would help smart buttons to find all the loaded buttons and make
    icon adjustments.

    Returns:
        (list[pyrevit.coreutils.ribbon._PyRevitRibbonButton]):
            list of ui button objects
    """
    from pyrevit.coreutils.ribbon import get_current_ui
    pyrvt_tabs = get_current_ui().get_pyrevit_tabs()
    buttons = []
    for tab in pyrvt_tabs:
        button = tab.find_child(EXEC_PARAMS.command_name)
        if button:
            buttons.append(button)
    return buttons


def toggle_icon(new_state, on_icon_path=None, off_icon_path=None, icon_size=ICON_MEDIUM):
    """Set the state of button icon (on or off).

    This method expects on.png and off.png in command bundle for on and off
    icon states, unless full path of icon states are provided.

    Args:
        new_state (bool): state of the ui button icon.
        on_icon_path (str, optional): full path of icon for on state.
                                      default='on.png'
        off_icon_path (str, optional): full path of icon for off state.
                                       default='off.png'
        icon_size (int): size of the icon. Defaults to medium (24x24)
    """
    # find the ui button
    uibuttons = get_all_buttons()
    if not uibuttons:
        mlogger.debug('Can not find ui button.')
        return

    # get icon for on state
    if not on_icon_path:
        on_icon_path = ui.resolve_icon_file(EXEC_PARAMS.command_path, exts.DEFAULT_ON_ICON_FILE)
        if not on_icon_path:
            mlogger.debug('Script does not have icon for on state.')
            return

    # get icon for off state
    if not off_icon_path:
        off_icon_path = ui.resolve_icon_file(EXEC_PARAMS.command_path, exts.DEFAULT_OFF_ICON_FILE)
        if not off_icon_path:
            mlogger.debug('Script does not have icon for on state.')
            return

    icon_path = on_icon_path if new_state else off_icon_path
    mlogger.debug('Setting icon state to: %s (%s)',
                  new_state, icon_path)

    for uibutton in uibuttons:
        uibutton.set_icon(icon_path, icon_size)


def exit():     #pylint: disable=W0622
    """Stop the script execution and exit."""
    sys.exit()


def show_file_in_explorer(file_path):
    """Show file in Windows Explorer."""
    coreutils.show_entry_in_explorer(file_path)


def show_folder_in_explorer(folder_path):
    """Show folder in Windows Explorer."""
    coreutils.open_folder_in_explorer(folder_path)


def open_url(url):
    """Open url in a new tab in default webbrowser."""
    import webbrowser
    if re.match('^https*://', url.lower()):
        webbrowser.open_new_tab(url)
    else:
        webbrowser.open_new_tab('http://' + url)


def clipboard_copy(string_to_copy):
    """Copy string to Windows Clipboard."""
    framework.Clipboard.SetText(string_to_copy)


def load_index(index_file='index.html'):
    """Load html file into output window.

    This method expects index.html file in the current command bundle,
    unless full path to an html file is provided.

    Args:
        index_file (str, optional): full path of html file.
    """
    outputwindow = get_output()
    if not op.isfile(index_file):
        index_file = get_bundle_file(index_file)
    outputwindow.open_page(index_file)


def load_ui(ui_instance, ui_file='ui.xaml', handle_esc=True, set_owner=True):
    """Load xaml file into given window instance.

    If window instance defines a method named `setup` it
    will be called after loading

    Args:
        ui_instance (forms.WPFWindow): ui form instance
        ui_file (str, optional): name of ui xaml file. defaults to: 'ui.xaml'
        handle_esc (bool, optional): handle escape and close window
        set_owner (bool, optional): set window owner to Revit window
    """
    ui_file = get_bundle_file(ui_file)
    if ui_file:
        ui_instance.load_xaml(
            ui_file,
            literal_string=False,
            handle_esc=handle_esc,
            set_owner=set_owner
            )
        if hasattr(ui_instance, 'setup'):
            ui_instance.setup()
        return ui_instance

    raise PyRevitException("Missing bundle ui file: {}".format(ui_file))


def get_envvar(envvar):
    """Return value of give pyRevit environment variable.

    The environment variable system is used to retain small values in memory
    between script runs (e.g. active/inactive state for toggle tools). Do not
    store large objects in memory using this method. List of currently set
    environment variables could be sees in pyRevit settings window.

    Args:
        envvar (str): name of environment variable

    Returns:
        (Any): object stored in environment variable

    Examples:
        ```python
        script.get_envvar('ToolActiveState')
        ```
        True
    """
    return envvars.get_pyrevit_env_var(envvar)


def set_envvar(envvar, value):
    """Set value of give pyRevit environment variable.

    The environment variable system is used to retain small values in memory
    between script runs (e.g. active/inactive state for toggle tools). Do not
    store large objects in memory using this method. List of currently set
    environment variables could be sees in pyRevit settings window.

    Args:
        envvar (str): name of environment variable
        value (any): value of environment variable

    Examples:
        ```python
        script.set_envvar('ToolActiveState', False)
        script.get_envvar('ToolActiveState')
        ```
        False
    """
    return envvars.set_pyrevit_env_var(envvar, value)


def dump_json(data, filepath):
    """Dumps given data into given json file.

    Args:
        data (object): serializable data to be dumped
        filepath (str): json file path
    """
    json_repr = json.dumps(data, indent=4, ensure_ascii=False)
    with codecs.open(filepath, 'w', "utf-8") as json_file:
        json_file.write(json_repr)


def load_json(filepath):
    """Loads data from given json file.

    Args:
        filepath (str): json file path

    Returns:
        (object): deserialized data
    """
    with codecs.open(filepath, 'r', "utf-8") as json_file:
        return json.load(json_file)


def dump_csv(data, filepath):
    """Dumps given data into given csv file.

    Args:
        data (list[list[str]]): data to be dumped
        filepath (str): csv file path
    """
    with codecs.open(filepath, 'wb', encoding='utf-8') as csvfile:
        writer = csv.writer(csvfile, delimiter=',', quotechar='\"')
        writer.writerows(data)


def load_csv(filepath):
    """Read lines from given csv file.

    Args:
        filepath (str): csv file path

    Returns:
        (list[list[str]]): csv data
    """
    with codecs.open(filepath, 'rb', encoding='utf-8') as csvfile:
        return list(csv.reader(csvfile, delimiter=',', quotechar='\"'))


def store_data(slot_name, data, this_project=True):
    """Wraps python pickle.dump() to easily store data to pyRevit data files.

    To store native Revit objects, use revit.serialize(). See Example

    Args:
        slot_name (type): desc
        data (obj): any pickalable data
        this_project (bool): data belongs to this project only

    Examples:
        ```python
        from pyrevit import revit
            from pyrevit import script


            class CustomData(object):
                def __init__(self, count, element_ids):
                    self._count = count
                    # serializes the Revit native objects
                    self._elmnt_ids = [revit.serialize(x) for x in element_ids]

                @property
                def count(self):
                    return self._count

                @property
                def element_ids(self):
                    # de-serializes the Revit native objects
                    return [x.deserialize() for x in self._elmnt_ids]


            mydata = CustomData(
                count=3,
                element_ids=[<DB.ElementId>, <DB.ElementId>, <DB.ElementId>]
            )

            script.store_data("Selected Elements", mydata)
        ```

    """
    # for this specific project?
    if this_project:
        data_file = get_document_data_file(file_id=slot_name,
                                           file_ext=DATAFEXT,
                                           add_cmd_name=False)
    # for any project file
    else:
        data_file = get_data_file(file_id=slot_name,
                                  file_ext=DATAFEXT,
                                  add_cmd_name=False)

    with open(data_file, "wb") as dfile:
        pickle.dump(data, dfile)


def load_data(slot_name, this_project=True):
    """Wraps python pickle.load() to easily load data from pyRevit data files.

    To recover native Revit objects, use revit.deserialize(). See Example

    Similar to pickle module, the custom data types must be defined in the main
    scope so the loader can create an instance and return original stored data

    Args:
        slot_name (type): desc
        this_project (bool): data belongs to this project only

    Returns:
        (object): stored data

    Examples:
        ```python
        from pyrevit import revit
        from pyrevit import script


            class CustomData(object):
                def __init__(self, count, element_ids):
                    self._count = count
                    # serializes the Revit native objects
                    self._elmnt_ids = [revit.serialize(x) for x in element_ids]

                @property
                def count(self):
                    return self._count

                @property
                def element_ids(self):
                    # de-serializes the Revit native objects
                    return [x.deserialize() for x in self._elmnt_ids]


            mydata = script.load_data("Selected Elements")
            mydata.element_ids
        ```
        [<DB.ElementId>, <DB.ElementId>, <DB.ElementId>]
    """
    # for this specific project?
    if this_project:
        data_file = get_document_data_file(file_id=slot_name,
                                           file_ext=DATAFEXT,
                                           add_cmd_name=False)
    # for any project file
    else:
        data_file = get_data_file(file_id=slot_name,
                                  file_ext=DATAFEXT,
                                  add_cmd_name=False)

    with open(data_file, 'rb') as dfile:
        return pickle.load(dfile)


def data_exists(slot_name, this_project=True):
    """Checks if data file in a specified slot and for certain project exists.

    Args:
        slot_name (type): desc
        this_project (bool): data belongs to this project only

    Returns:
        (bool): true if the path exists
    """
    # for this specific project?
    if this_project:
        data_file = get_document_data_file(file_id=slot_name,
                                           file_ext=DATAFEXT,
                                           add_cmd_name=False)
    # for any project file
    else:
        data_file = get_data_file(file_id=slot_name,
                                  file_ext=DATAFEXT,
                                  add_cmd_name=False)
    return os.path.exists(data_file)



class EventSourceCreationData(object):
 """
 Represents the configuration settings used to create an event log source on the local computer or a remote computer.

 

 EventSourceCreationData(source: str,logName: str)
 """
 @staticmethod
 def __new__(self,source,logName):
  """ __new__(cls: type,source: str,logName: str) """
  pass
 CategoryCount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the number of categories in the category resource file.



Get: CategoryCount(self: EventSourceCreationData) -> int



Set: CategoryCount(self: EventSourceCreationData)=value

"""

 CategoryResourceFile=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the path of the resource file that contains category strings for the source.



Get: CategoryResourceFile(self: EventSourceCreationData) -> str



Set: CategoryResourceFile(self: EventSourceCreationData)=value

"""

 LogName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the event log to which the source writes entries.



Get: LogName(self: EventSourceCreationData) -> str



Set: LogName(self: EventSourceCreationData)=value

"""

 MachineName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the computer on which to register the event source.



Get: MachineName(self: EventSourceCreationData) -> str



Set: MachineName(self: EventSourceCreationData)=value

"""

 MessageResourceFile=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the path of the message resource file that contains message formatting strings for the source.



Get: MessageResourceFile(self: EventSourceCreationData) -> str



Set: MessageResourceFile(self: EventSourceCreationData)=value

"""

 ParameterResourceFile=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the path of the resource file that contains message parameter strings for the source.



Get: ParameterResourceFile(self: EventSourceCreationData) -> str



Set: ParameterResourceFile(self: EventSourceCreationData)=value

"""

 Source=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name to register with the event log as an event source.



Get: Source(self: EventSourceCreationData) -> str



Set: Source(self: EventSourceCreationData)=value

"""





class StringWriter(TextWriter,IDisposable):
 """
 Implements a System.IO.TextWriter for writing information to a string. The information is stored in an underlying System.Text.StringBuilder.

 

 StringWriter()

 StringWriter(formatProvider: IFormatProvider)

 StringWriter(sb: StringBuilder)

 StringWriter(sb: StringBuilder,formatProvider: IFormatProvider)
 """
 def Close(self):
  """
  Close(self: StringWriter)

   Closes the current System.IO.StringWriter and the underlying stream.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: StringWriter,disposing: bool)

   Releases the unmanaged resources used by the System.IO.StringWriter and optionally releases the 

    managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def FlushAsync(self):
  """ FlushAsync(self: StringWriter) -> Task """
  pass
 def GetStringBuilder(self):
  """
  GetStringBuilder(self: StringWriter) -> StringBuilder

  

   Returns the underlying System.Text.StringBuilder.

   Returns: The underlying StringBuilder.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def ToString(self):
  """
  ToString(self: StringWriter) -> str

  

   Returns a string containing the characters written to the current StringWriter so far.

   Returns: The string containing the characters written to the current StringWriter.
  """
  pass
 def Write(self,*__args):
  """
  Write(self: StringWriter,value: str)

   Writes a string to this instance of the StringWriter.

  

   value: The string to write.

  Write(self: StringWriter,buffer: Array[Char],index: int,count: int)

   Writes the specified region of a character array to this instance of the StringWriter.

  

   buffer: The character array to read data from.

   index: The index at which to begin reading from buffer.

   count: The maximum number of characters to write.

  Write(self: StringWriter,value: Char)

   Writes a character to this instance of the StringWriter.

  

   value: The character to write.
  """
  pass
 def WriteAsync(self,*__args):
  """
  WriteAsync(self: StringWriter,buffer: Array[Char],index: int,count: int) -> Task

  WriteAsync(self: StringWriter,value: str) -> Task

  WriteAsync(self: StringWriter,value: Char) -> Task
  """
  pass
 def WriteLineAsync(self,*__args):
  """
  WriteLineAsync(self: StringWriter,buffer: Array[Char],index: int,count: int) -> Task

  WriteLineAsync(self: StringWriter,value: str) -> Task

  WriteLineAsync(self: StringWriter,value: Char) -> Task
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,formatProvider: IFormatProvider)

  __new__(cls: type,sb: StringBuilder)

  __new__(cls: type,sb: StringBuilder,formatProvider: IFormatProvider)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Encoding=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Text.Encoding in which the output is written.



Get: Encoding(self: StringWriter) -> Encoding



"""


 CoreNewLine=None




class ICancelAddNew:
 """ Adds transactional capability when adding a new item to a collection. """
 def CancelNew(self,itemIndex):
  """
  CancelNew(self: ICancelAddNew,itemIndex: int)

   Discards a pending new item from the collection.

  

   itemIndex: The index of the item that was previously added to the collection.
  """
  pass
 def EndNew(self,itemIndex):
  """
  EndNew(self: ICancelAddNew,itemIndex: int)

   Commits a pending new item to the collection.

  

   itemIndex: The index of the item that was previously added to the collection.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class UnmanagedMemoryAccessor(object,IDisposable):
 """
 Provides random access to unmanaged blocks of memory from managed code.

 

 UnmanagedMemoryAccessor(buffer: SafeBuffer,offset: Int64,capacity: Int64)

 UnmanagedMemoryAccessor(buffer: SafeBuffer,offset: Int64,capacity: Int64,access: FileAccess)
 """
 def Dispose(self):
  """
  Dispose(self: UnmanagedMemoryAccessor)

   Releases all resources used by the System.IO.UnmanagedMemoryAccessor.
  """
  pass
 def Initialize(self,*args):
  """
  Initialize(self: UnmanagedMemoryAccessor,buffer: SafeBuffer,offset: Int64,capacity: Int64,access: FileAccess)

   Sets the initial values for the accessor.

  

   buffer: The buffer to contain the accessor.

   offset: The byte at which to start the accessor.

   capacity: The size,in bytes,of memory to allocate.

   access: The type of access allowed to the memory. The default is 

    System.IO.MemoryMappedFiles.MemoryMappedFileAccess.ReadWrite.
  """
  pass
 def Read(self,position,structure):
  """ Read[T](self: UnmanagedMemoryAccessor,position: Int64) -> T """
  pass
 def ReadArray(self,position,array,offset,count):
  """ ReadArray[T](self: UnmanagedMemoryAccessor,position: Int64,array: Array[T],offset: int,count: int) -> int """
  pass
 def ReadBoolean(self,position):
  """
  ReadBoolean(self: UnmanagedMemoryAccessor,position: Int64) -> bool

  

   Reads a Boolean value from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: true or false.
  """
  pass
 def ReadByte(self,position):
  """
  ReadByte(self: UnmanagedMemoryAccessor,position: Int64) -> Byte

  

   Reads a byte value from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadChar(self,position):
  """
  ReadChar(self: UnmanagedMemoryAccessor,position: Int64) -> Char

  

   Reads a character from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadDecimal(self,position):
  """
  ReadDecimal(self: UnmanagedMemoryAccessor,position: Int64) -> Decimal

  

   Reads a decimal value from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadDouble(self,position):
  """
  ReadDouble(self: UnmanagedMemoryAccessor,position: Int64) -> float

  

   Reads a double-precision floating-point value from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadInt16(self,position):
  """
  ReadInt16(self: UnmanagedMemoryAccessor,position: Int64) -> Int16

  

   Reads a 16-bit integer from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadInt32(self,position):
  """
  ReadInt32(self: UnmanagedMemoryAccessor,position: Int64) -> int

  

   Reads a 32-bit integer from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadInt64(self,position):
  """
  ReadInt64(self: UnmanagedMemoryAccessor,position: Int64) -> Int64

  

   Reads a 64-bit integer from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadSByte(self,position):
  """
  ReadSByte(self: UnmanagedMemoryAccessor,position: Int64) -> SByte

  

   Reads an 8-bit signed integer from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadSingle(self,position):
  """
  ReadSingle(self: UnmanagedMemoryAccessor,position: Int64) -> Single

  

   Reads a single-precision floating-point value from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadUInt16(self,position):
  """
  ReadUInt16(self: UnmanagedMemoryAccessor,position: Int64) -> UInt16

  

   Reads an unsigned 16-bit integer from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadUInt32(self,position):
  """
  ReadUInt32(self: UnmanagedMemoryAccessor,position: Int64) -> UInt32

  

   Reads an unsigned 32-bit integer from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def ReadUInt64(self,position):
  """
  ReadUInt64(self: UnmanagedMemoryAccessor,position: Int64) -> UInt64

  

   Reads an unsigned 64-bit integer from the accessor.

  

   position: The number of bytes into the accessor at which to begin reading.

   Returns: The value that was read.
  """
  pass
 def Write(self,position,*__args):
  """
  Write(self: UnmanagedMemoryAccessor,position: Int64,value: SByte)

   Writes an 8-bit integer into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: float)

   Writes a Double value into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: Single)

   Writes a Single into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: UInt16)

   Writes an unsigned 16-bit integer into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write[T](self: UnmanagedMemoryAccessor,position: Int64,structure: T) -> T

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: UInt64)

   Writes an unsigned 64-bit integer into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: UInt32)

   Writes an unsigned 32-bit integer into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: Char)

   Writes a character into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: Byte)

   Writes a byte value into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: bool)

   Writes a Boolean value into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: Int16)

   Writes a 16-bit integer into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: Decimal)

   Writes a decimal value into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: Int64)

   Writes a 64-bit integer into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.

  Write(self: UnmanagedMemoryAccessor,position: Int64,value: int)

   Writes a 32-bit integer into the accessor.

  

   position: The number of bytes into the accessor at which to begin writing.

   value: The value to write.
  """
  pass
 def WriteArray(self,position,array,offset,count):
  """ WriteArray[T](self: UnmanagedMemoryAccessor,position: Int64,array: Array[T],offset: int,count: int) """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,buffer,offset,capacity,access=None):
  """
  __new__(cls: type)

  __new__(cls: type,buffer: SafeBuffer,offset: Int64,capacity: Int64)

  __new__(cls: type,buffer: SafeBuffer,offset: Int64,capacity: Int64,access: FileAccess)
  """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 CanRead=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Determines whether the accessor is readable.



Get: CanRead(self: UnmanagedMemoryAccessor) -> bool



"""

 CanWrite=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Determines whether the accessory is writable.



Get: CanWrite(self: UnmanagedMemoryAccessor) -> bool



"""

 Capacity=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the capacity of the accessor.



Get: Capacity(self: UnmanagedMemoryAccessor) -> Int64



"""

 IsOpen=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Determines whether the accessor is currently open by a process.



"""





class DataGridColumnStyle(Component,IComponent,IDisposable,IDataGridColumnStyleEditingNotificationService):
 """
 Specifies the appearance,text formatting,and behavior of a System.Windows.Forms.DataGrid control column. This class is abstract.

 

 DataGridColumnStyle()

 DataGridColumnStyle(prop: PropertyDescriptor)
 """
 def Abort(self,*args):
  """
  Abort(self: DataGridColumnStyle,rowNum: int)

   When overridden in a derived class,initiates a request to interrupt an edit procedure.

  

   rowNum: The row number upon which an operation is being interrupted.
  """
  pass
 def BeginUpdate(self,*args):
  """
  BeginUpdate(self: DataGridColumnStyle)

   Suspends the painting of the column until the System.Windows.Forms.DataGridColumnStyle.EndUpdate 

    method is called.
  """
  pass
 def CheckValidDataSource(self,*args):
  """
  CheckValidDataSource(self: DataGridColumnStyle,value: CurrencyManager)

   Throws an exception if the System.Windows.Forms.DataGrid does not have a valid data source,or 

    if this column is not mapped to a valid property in the data source.

  

  

   value: A System.Windows.Forms.CurrencyManager to check.
  """
  pass
 def ColumnStartedEditing(self,*args):
  """
  ColumnStartedEditing(self: DataGridColumnStyle,editingControl: Control)

   Informs the System.Windows.Forms.DataGrid that the user has begun editing the column.

  

   editingControl: The System.Windows.Forms.Control that hosted by the column.
  """
  pass
 def Commit(self,*args):
  """
  Commit(self: DataGridColumnStyle,dataSource: CurrencyManager,rowNum: int) -> bool

  

   When overridden in a derived class,initiates a request to complete an editing procedure.

  

   dataSource: The System.Windows.Forms.CurrencyManager for the System.Windows.Forms.DataGridColumnStyle.

   rowNum: The number of the row being edited.

   Returns: true if the editing procedure committed successfully; otherwise,false.
  """
  pass
 def ConcedeFocus(self,*args):
  """
  ConcedeFocus(self: DataGridColumnStyle)

   Notifies a column that it must relinquish the focus to the control it is hosting.
  """
  pass
 def CreateHeaderAccessibleObject(self,*args):
  """
  CreateHeaderAccessibleObject(self: DataGridColumnStyle) -> AccessibleObject

  

   Gets the System.Windows.Forms.AccessibleObject for the column.

   Returns: An System.Windows.Forms.AccessibleObject for the column.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: Component,disposing: bool)

   Releases the unmanaged resources used by the System.ComponentModel.Component and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def Edit(self,*args):
  """
  Edit(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int,bounds: Rectangle,readOnly: bool,displayText: str,cellIsVisible: bool)

   When overridden in a deriving class,prepares a cell for editing.

  

   source: The System.Windows.Forms.CurrencyManager for the System.Windows.Forms.DataGridColumnStyle.

   rowNum: The row number in this column which is being edited.

   bounds: The System.Drawing.Rectangle in which the control is to be sited.

   readOnly: A value indicating whether the column is a read-only. true if the value is read-only; otherwise,

    false.

  

   displayText: The text to display in the control.

   cellIsVisible: A value indicating whether the cell is visible. true if the cell is visible; otherwise,false.

  Edit(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int,bounds: Rectangle,readOnly: bool,displayText: str)

   Prepares the cell for editing using the specified System.Windows.Forms.CurrencyManager,row 

    number,and System.Drawing.Rectangle parameters.

  

  

   source: The System.Windows.Forms.CurrencyManager for the System.Windows.Forms.DataGridColumnStyle.

   rowNum: The row number in this column which is being edited.

   bounds: The System.Drawing.Rectangle in which the control is to be sited.

   readOnly: A value indicating whether the column is a read-only. true if the value is read-only; otherwise,

    false.

  

   displayText: The text to display in the control.

  Edit(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int,bounds: Rectangle,readOnly: bool)

   Prepares a cell for editing.

  

   source: The System.Windows.Forms.CurrencyManager for the System.Windows.Forms.DataGridColumnStyle.

   rowNum: The row number to edit.

   bounds: The bounding System.Drawing.Rectangle in which the control is to be sited.

   readOnly: A value indicating whether the column is a read-only. true if the value is read-only; otherwise,

    false.
  """
  pass
 def EndUpdate(self,*args):
  """
  EndUpdate(self: DataGridColumnStyle)

   Resumes the painting of columns suspended by calling the 

    System.Windows.Forms.DataGridColumnStyle.BeginUpdate method.
  """
  pass
 def EnterNullValue(self,*args):
  """
  EnterNullValue(self: DataGridColumnStyle)

   Enters a System.DBNull.Value into the column.
  """
  pass
 def GetColumnValueAtRow(self,*args):
  """
  GetColumnValueAtRow(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int) -> object

  

   Gets the value in the specified row from the specified System.Windows.Forms.CurrencyManager.

  

   source: The System.Windows.Forms.CurrencyManager containing the data.

   rowNum: The row number containing the data.

   Returns: An System.Object containing the value.
  """
  pass
 def GetMinimumHeight(self,*args):
  """
  GetMinimumHeight(self: DataGridColumnStyle) -> int

  

   When overridden in a derived class,gets the minimum height of a row.

   Returns: The minimum height of a row.
  """
  pass
 def GetPreferredHeight(self,*args):
  """
  GetPreferredHeight(self: DataGridColumnStyle,g: Graphics,value: object) -> int

  

   When overridden in a derived class,gets the height used for automatically resizing columns.

  

   g: A System.Drawing.Graphics object.

   value: An object value for which you want to know the screen height and width.

   Returns: The height used for auto resizing a cell.
  """
  pass
 def GetPreferredSize(self,*args):
  """
  GetPreferredSize(self: DataGridColumnStyle,g: Graphics,value: object) -> Size

  

   When overridden in a derived class,gets the width and height of the specified value. The width 

    and height are used when the user navigates to System.Windows.Forms.DataGridTableStyle using the 

    System.Windows.Forms.DataGridColumnStyle.

  

  

   g: A System.Drawing.Graphics object.

   value: An object value for which you want to know the screen height and width.

   Returns: A System.Drawing.Size that contains the dimensions of the cell.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def Invalidate(self,*args):
  """
  Invalidate(self: DataGridColumnStyle)

   Redraws the column and causes a paint message to be sent to the control.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def Paint(self,*args):
  """
  Paint(self: DataGridColumnStyle,g: Graphics,bounds: Rectangle,source: CurrencyManager,rowNum: int,backBrush: Brush,foreBrush: Brush,alignToRight: bool)

   Paints a System.Windows.Forms.DataGridColumnStyle with the specified System.Drawing.Graphics,

    System.Drawing.Rectangle,System.Windows.Forms.CurrencyManager,row number,background color,

    foreground color,and alignment.

  

  

   g: The System.Drawing.Graphics to draw to.

   bounds: The bounding System.Drawing.Rectangle to paint into.

   source: The System.Windows.Forms.CurrencyManager of the System.Windows.Forms.DataGrid control the column 

    belongs to.

  

   rowNum: The number of the row in the underlying data table being referred to.

   backBrush: A System.Drawing.Brush used to paint the background color.

   foreBrush: A System.Drawing.Color used to paint the foreground color.

   alignToRight: A value indicating whether to align the content to the right. true if the content is aligned to 

    the right,otherwise,false.

  

  Paint(self: DataGridColumnStyle,g: Graphics,bounds: Rectangle,source: CurrencyManager,rowNum: int,alignToRight: bool)

   When overridden in a derived class,paints a System.Windows.Forms.DataGridColumnStyle with the 

    specified System.Drawing.Graphics,System.Drawing.Rectangle,

    System.Windows.Forms.CurrencyManager,row number,and alignment.

  

  

   g: The System.Drawing.Graphics to draw to.

   bounds: The bounding System.Drawing.Rectangle to paint into.

   source: The System.Windows.Forms.CurrencyManager of the System.Windows.Forms.DataGrid control the column 

    belongs to.

  

   rowNum: The number of the row in the underlying data being referred to.

   alignToRight: A value indicating whether to align the column's content to the right. true if the content 

    should be aligned to the right; otherwise false.

  

  Paint(self: DataGridColumnStyle,g: Graphics,bounds: Rectangle,source: CurrencyManager,rowNum: int)

   Paints the System.Windows.Forms.DataGridColumnStyle with the specified System.Drawing.Graphics,

    System.Drawing.Rectangle,System.Windows.Forms.CurrencyManager,and row number.

  

  

   g: The System.Drawing.Graphics to draw to.

   bounds: The bounding System.Drawing.Rectangle to paint into.

   source: The System.Windows.Forms.CurrencyManager of the System.Windows.Forms.DataGrid control the column 

    belongs to.

  

   rowNum: The number of the row in the underlying data being referred to.
  """
  pass
 def ReleaseHostedControl(self,*args):
  """
  ReleaseHostedControl(self: DataGridColumnStyle)

   Allows the column to free resources when the control it hosts is not needed.
  """
  pass
 def ResetHeaderText(self):
  """
  ResetHeaderText(self: DataGridColumnStyle)

   Resets the System.Windows.Forms.DataGridColumnStyle.HeaderText to its default value,null.
  """
  pass
 def SetColumnValueAtRow(self,*args):
  """
  SetColumnValueAtRow(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int,value: object)

   Sets the value in a specified row with the value from a specified 

    System.Windows.Forms.CurrencyManager.

  

  

   source: A System.Windows.Forms.CurrencyManager associated with the 

    System.Windows.Forms.DataGridColumnStyle.

  

   rowNum: The number of the row.

   value: The value to set.
  """
  pass
 def SetDataGrid(self,*args):
  """
  SetDataGrid(self: DataGridColumnStyle,value: DataGrid)

   Sets the System.Windows.Forms.DataGrid control that this column belongs to.

  

   value: The System.Windows.Forms.DataGrid control that this column belongs to.
  """
  pass
 def SetDataGridInColumn(self,*args):
  """
  SetDataGridInColumn(self: DataGridColumnStyle,value: DataGrid)

   Sets the System.Windows.Forms.DataGrid for the column.

  

   value: A System.Windows.Forms.DataGrid.
  """
  pass
 def UpdateUI(self,*args):
  """
  UpdateUI(self: DataGridColumnStyle,source: CurrencyManager,rowNum: int,displayText: str)

   Updates the value of a specified row with the given text.

  

   source: The System.Windows.Forms.CurrencyManager associated with the 

    System.Windows.Forms.DataGridColumnStyle.

  

   rowNum: The row to update.

   displayText: The new value.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,prop=None):
  """
  __new__(cls: type)

  __new__(cls: type,prop: PropertyDescriptor)
  """
  pass
 def __str__(self,*args):
  pass
 Alignment=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the alignment of text in a column.



Get: Alignment(self: DataGridColumnStyle) -> HorizontalAlignment



Set: Alignment(self: DataGridColumnStyle)=value

"""

 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 DataGridTableStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.DataGridTableStyle for the column.



Get: DataGridTableStyle(self: DataGridColumnStyle) -> DataGridTableStyle



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 FontHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of the column's font.



"""

 HeaderAccessibleObject=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.AccessibleObject for the column.



Get: HeaderAccessibleObject(self: DataGridColumnStyle) -> AccessibleObject



"""

 HeaderText=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the text of the column header.



Get: HeaderText(self: DataGridColumnStyle) -> str



Set: HeaderText(self: DataGridColumnStyle)=value

"""

 MappingName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the data member to map the column style to.



Get: MappingName(self: DataGridColumnStyle) -> str



Set: MappingName(self: DataGridColumnStyle)=value

"""

 NullText=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the text that is displayed when the column contains null.



Get: NullText(self: DataGridColumnStyle) -> str



Set: NullText(self: DataGridColumnStyle)=value

"""

 PropertyDescriptor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.ComponentModel.PropertyDescriptor that determines the attributes of data displayed by the System.Windows.Forms.DataGridColumnStyle.



Get: PropertyDescriptor(self: DataGridColumnStyle) -> PropertyDescriptor



Set: PropertyDescriptor(self: DataGridColumnStyle)=value

"""

 ReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the data in the column can be edited.



Get: ReadOnly(self: DataGridColumnStyle) -> bool



Set: ReadOnly(self: DataGridColumnStyle)=value

"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the width of the column.



Get: Width(self: DataGridColumnStyle) -> int



Set: Width(self: DataGridColumnStyle)=value

"""


 AlignmentChanged=None
 CompModSwitches=None
 DataGridColumnHeaderAccessibleObject=None
 FontChanged=None
 HeaderTextChanged=None
 MappingNameChanged=None
 NullTextChanged=None
 PropertyDescriptorChanged=None
 ReadOnlyChanged=None
 WidthChanged=None




"""Create necessary compiled types for pyRevit bundles."""
from pyrevit import coreutils
from pyrevit.coreutils import logger
import pyrevit.extensions as exts

from pyrevit import runtime


#pylint: disable=W0703,C0302,C0103
mlogger = logger.get_logger(__name__)


def create_bundle_type(
        module_builder,
        type_name,
        bundle_script,
        bundle_config_script,
        bundle_search_paths,
        bundle_arguments,
        bundle_help_url,
        bundle_tooltip,
        bundle_name,
        bundle_full_name,
        bundle_extension_name,
        bundle_unique_name,
        bundle_control_id,
        bundle_context,
        engine_cfgs,
    ):
    runtime.create_type(
        module_builder,
        runtime.CMD_EXECUTOR_TYPE,
        type_name,
        runtime.create_ext_command_attrs(),
        bundle_script,
        bundle_config_script,
        coreutils.join_strings(bundle_search_paths),
        coreutils.join_strings(bundle_arguments),
        bundle_help_url,
        bundle_tooltip,
        bundle_name,
        bundle_full_name,
        bundle_extension_name,
        bundle_unique_name,
        bundle_control_id,
        bundle_context,
        engine_cfgs)


def create_executor_type(extension, module_builder, cmd_component, eng_cfgs=''):
    mlogger.debug('Creating executor type for: %s', cmd_component)
    mlogger.debug('%s uses clean engine: %s',
                  cmd_component.name, cmd_component.requires_clean_engine)
    mlogger.debug('%s requires Fullframe engine: %s',
                  cmd_component.name, cmd_component.requires_fullframe_engine)
    mlogger.debug('%s requires Fullframe engine: %s',
                  cmd_component.name, cmd_component.requires_fullframe_engine)

    create_bundle_type(
        module_builder=module_builder,
        type_name=cmd_component.unique_name,
        bundle_script=cmd_component.script_file or "",
        bundle_config_script=cmd_component.config_script_file or "",
        bundle_search_paths=cmd_component.module_paths,
        bundle_arguments=cmd_component.arguments,
        bundle_help_url=cmd_component.help_url or "",
        bundle_tooltip=cmd_component.tooltip or "",
        bundle_name=cmd_component.name,
        bundle_full_name=cmd_component.get_full_bundle_name(),
        bundle_extension_name=extension.name,
        bundle_unique_name=cmd_component.unique_name,
        bundle_control_id=cmd_component.control_id,
        bundle_context=cmd_component.context or "",
        engine_cfgs=eng_cfgs
        )

    mlogger.debug('Successfully created executor type for: %s', cmd_component)


def create_selection_avail_type(module_builder, cmd_component):
    runtime.create_type(module_builder,
                        runtime.CMD_AVAIL_TYPE_SELECTION,
                        cmd_component.avail_class_name,
                        [],
                        cmd_component.context)


def create_zerodoc_avail_type(module_builder, cmd_component):
    runtime.create_type(module_builder,
                        runtime.CMD_AVAIL_TYPE_ZERODOC,
                        cmd_component.avail_class_name,
                        [])


def create_extended_avail_type(module_builder, cmd_component):
    runtime.create_type(module_builder,
                        runtime.CMD_AVAIL_TYPE_EXTENDED,
                        cmd_component.avail_class_name,
                        [],
                        cmd_component.context)



class FileSystemSecurity(NativeObjectSecurity):
 """ Represents the access control and audit security for a file or directory. """
 def AccessRuleFactory(self,identityReference,accessMask,isInherited,inheritanceFlags,propagationFlags,type):
  """
  AccessRuleFactory(self: FileSystemSecurity,identityReference: IdentityReference,accessMask: int,isInherited: bool,inheritanceFlags: InheritanceFlags,propagationFlags: PropagationFlags,type: AccessControlType) -> AccessRule

  

   Initializes a new instance of the System.Security.AccessControl.FileSystemAccessRule class that 

    represents a new access control rule for the specified user,with the specified access rights,

    access control,and flags.

  

  

   identityReference: An System.Security.Principal.IdentityReference object that represents a user account.

   accessMask: An integer that specifies an access type.

   isInherited: true if the access rule is inherited; otherwise,false.

   inheritanceFlags: One of the System.Security.AccessControl.InheritanceFlags values that specifies how to propagate 

    access masks to child objects.

  

   propagationFlags: One of the System.Security.AccessControl.PropagationFlags values that specifies how to propagate 

    Access Control Entries (ACEs) to child objects.

  

   type: One of the System.Security.AccessControl.AccessControlType values that specifies whether access 

    is allowed or denied.

  

   Returns: A new System.Security.AccessControl.FileSystemAccessRule object that represents a new access 

    control rule for the specified user,with the specified access rights,access control,and 

    flags.
  """
  pass
 def AddAccessRule(self,rule):
  """
  AddAccessRule(self: FileSystemSecurity,rule: FileSystemAccessRule)

   Adds the specified access control list (ACL) permission to the current file or directory.

  

   rule: A System.Security.AccessControl.FileSystemAccessRule object that represents an access control 

    list (ACL) permission to add to a file or directory.
  """
  pass
 def AddAuditRule(self,rule):
  """
  AddAuditRule(self: FileSystemSecurity,rule: FileSystemAuditRule)

   Adds the specified audit rule to the current file or directory.

  

   rule: A System.Security.AccessControl.FileSystemAuditRule  object that represents an audit rule to add 

    to a file or directory.
  """
  pass
 def AuditRuleFactory(self,identityReference,accessMask,isInherited,inheritanceFlags,propagationFlags,flags):
  """
  AuditRuleFactory(self: FileSystemSecurity,identityReference: IdentityReference,accessMask: int,isInherited: bool,inheritanceFlags: InheritanceFlags,propagationFlags: PropagationFlags,flags: AuditFlags) -> AuditRule

  

   Initializes a new instance of the System.Security.AccessControl.FileSystemAuditRule class 

    representing the specified audit rule for the specified user.

  

  

   identityReference: An System.Security.Principal.IdentityReference object that represents a user account.

   accessMask: An integer that specifies an access type.

   isInherited: true if the access rule is inherited; otherwise,false.

   inheritanceFlags: One of the System.Security.AccessControl.InheritanceFlags values that specifies how to propagate 

    access masks to child objects.

  

   propagationFlags: One of the System.Security.AccessControl.PropagationFlags values that specifies how to propagate 

    Access Control Entries (ACEs) to child objects.

  

   flags: One of the System.Security.AccessControl.AuditFlags values that specifies the type of auditing 

    to perform.

  

   Returns: A new System.Security.AccessControl.FileSystemAuditRule object representing the specified audit 

    rule for the specified user.
  """
  pass
 def RemoveAccessRule(self,rule):
  """
  RemoveAccessRule(self: FileSystemSecurity,rule: FileSystemAccessRule) -> bool

  

   Removes all matching allow or deny access control list (ACL) permissions from the current file 

    or directory.

  

  

   rule: A System.Security.AccessControl.FileSystemAccessRule object that represents an access control 

    list (ACL) permission to remove from a file or directory.

  

   Returns: true if the access rule was removed; otherwise,false.
  """
  pass
 def RemoveAccessRuleAll(self,rule):
  """
  RemoveAccessRuleAll(self: FileSystemSecurity,rule: FileSystemAccessRule)

   Removes all access control list (ACL) permissions for the specified user from the current file 

    or directory.

  

  

   rule: A System.Security.AccessControl.FileSystemAccessRule object that specifies a user whose access 

    control list (ACL) permissions should be removed from a file or directory.
  """
  pass
 def RemoveAccessRuleSpecific(self,rule):
  """
  RemoveAccessRuleSpecific(self: FileSystemSecurity,rule: FileSystemAccessRule)

   Removes a single matching allow or deny access control list (ACL) permission from the current 

    file or directory.

  

  

   rule: A System.Security.AccessControl.FileSystemAccessRule object that specifies a user whose access 

    control list (ACL) permissions should be removed from a file or directory.
  """
  pass
 def RemoveAuditRule(self,rule):
  """
  RemoveAuditRule(self: FileSystemSecurity,rule: FileSystemAuditRule) -> bool

  

   Removes all matching allow or deny audit rules from the current file or directory.

  

   rule: A System.Security.AccessControl.FileSystemAuditRule  object that represents an audit rule to 

    remove from a file or directory.

  

   Returns: true if the audit rule was removed; otherwise,false
  """
  pass
 def RemoveAuditRuleAll(self,rule):
  """
  RemoveAuditRuleAll(self: FileSystemSecurity,rule: FileSystemAuditRule)

   Removes all audit rules for the specified user from the current file or directory.

  

   rule: A System.Security.AccessControl.FileSystemAuditRule object that specifies a user whose audit 

    rules should be removed from a file or directory.
  """
  pass
 def RemoveAuditRuleSpecific(self,rule):
  """
  RemoveAuditRuleSpecific(self: FileSystemSecurity,rule: FileSystemAuditRule)

   Removes a single matching allow or deny audit rule from the current file or directory.

  

   rule: A System.Security.AccessControl.FileSystemAuditRule  object that represents an audit rule to 

    remove from a file or directory.
  """
  pass
 def ResetAccessRule(self,rule):
  """
  ResetAccessRule(self: FileSystemSecurity,rule: FileSystemAccessRule)

   Adds the specified access control list (ACL) permission to the current file or directory and 

    removes all matching ACL permissions.

  

  

   rule: A System.Security.AccessControl.FileSystemAccessRule object that represents an access control 

    list (ACL) permission to add to a file or directory.
  """
  pass
 def SetAccessRule(self,rule):
  """
  SetAccessRule(self: FileSystemSecurity,rule: FileSystemAccessRule)

   Sets the specified access control list (ACL) permission for the current file or directory.

  

   rule: A System.Security.AccessControl.FileSystemAccessRule object that represents an access control 

    list (ACL) permission to set for a file or directory.
  """
  pass
 def SetAuditRule(self,rule):
  """
  SetAuditRule(self: FileSystemSecurity,rule: FileSystemAuditRule)

   Sets the specified audit rule for the current file or directory.

  

   rule: A System.Security.AccessControl.FileSystemAuditRule object that represents an audit rule to set 

    for a file or directory.
  """
  pass
 AccessRightType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the enumeration that the System.Security.AccessControl.FileSystemSecurity class uses to represent access rights.



Get: AccessRightType(self: FileSystemSecurity) -> Type



"""

 AccessRulesModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a Boolean value that specifies whether the access rules associated with this System.Security.AccessControl.ObjectSecurity object have been modified.



"""

 AccessRuleType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the enumeration that the System.Security.AccessControl.FileSystemSecurity class uses to represent access rules.



Get: AccessRuleType(self: FileSystemSecurity) -> Type



"""

 AuditRulesModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a Boolean value that specifies whether the audit rules associated with this System.Security.AccessControl.ObjectSecurity object have been modified.



"""

 AuditRuleType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the type that the System.Security.AccessControl.FileSystemSecurity class uses to represent audit rules.



Get: AuditRuleType(self: FileSystemSecurity) -> Type



"""

 GroupModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a Boolean value that specifies whether the group associated with the securable object has been modified.



"""

 IsContainer=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a Boolean value that specifies whether this System.Security.AccessControl.ObjectSecurity object is a container object.



"""

 IsDS=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a Boolean value that specifies whether this System.Security.AccessControl.ObjectSecurity object is a directory object.



"""

 OwnerModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a Boolean value that specifies whether the owner of the securable object has been modified.



"""





from rpw import DB
from rpw.base import BaseObjectWrapper
from rpw.exceptions import RpwCoerceError
from rpw.db.transform import Transform
from collections import OrderedDict

class XYZ(BaseObjectWrapper):
    """
    `DB.XYZ` Wrapper

    XYZ light wrapper with a few helpful methods:

    >>> from rpw.db import db
    >>> pt = db.XYZ(some_point)
    >>> pt.as_tuple
    (0,0,0)
    >>> pt.x = 10
    <rpw:XYZ % DB.XYZ: 0,0,10>
    >>> pt.at_z(5)
    <rpw:XYZ % DB.XYZ: 0,0,5>
    >>> pt.as_dict()
    {'x': 0, 'y':0, 'z':5}

    Attributes:
        _revit_object (DB.XYZ): Wrapped ``DB.XYZ``
    """

    _revit_object_class = DB.XYZ

    def __init__(self, *point_reference):
        """
        XYZ Supports a wide variety of instantiation overloads:

        >>> XYZ(0,0)
        >>> XYZ(0,0,0)
        >>> XYZ([0,0])
        >>> XYZ([0,0,0])
        >>> XYZ(DB.XYZ(0,0,0))

        Args:
            point_reference (``DB.XYZ``,``iterable``, ``args``): Point like data
        """
        # XYZ(0,0,0)
        if len(point_reference) == 3:
            xyz = DB.XYZ(*point_reference)
        # XYZ(0,0)
        elif len(point_reference) == 2:
            xyz = DB.XYZ(point_reference[0], point_reference[1], 0)
        # XYZ([0,0,0]) or # XYZ([0,0])
        elif len(point_reference) == 1 and isinstance(point_reference[0], (tuple, list)):
            # Assumes one arg, tuple
            xyz = XYZ(*point_reference[0])
            xyz = DB.XYZ(*xyz.as_tuple)
        # XYZ(DB.XYZ(0,0,0))
        elif len(point_reference) == 1 and isinstance(point_reference[0], DB.XYZ):
            # Assumes one arg, DB.XYZ
            xyz = point_reference[0]
        elif len(point_reference) == 1 and isinstance(point_reference[0], XYZ):
            # Assumes one arg, DB.XYZ
            xyz = point_reference[0].unwrap()
        else:
            raise RpwCoerceError(point_reference, 'point-like object')
        super(XYZ, self).__init__(xyz)

    @property
    def x(self):
        """X Value"""
        return self._revit_object.X

    @property
    def y(self):
        """Y Value"""
        return self._revit_object.Y

    @property
    def z(self):
        """Z Value"""
        return self._revit_object.Z

    @x.setter
    def x(self, value):
        self._revit_object = DB.XYZ(value, self.y, self.z)

    @y.setter
    def y(self, value):
        self._revit_object = DB.XYZ(self.x, value, self.z)

    @z.setter
    def z(self, value):
        self._revit_object = DB.XYZ(self.x, self.y, value)

    def at_z(self, z, wrapped=True):
        """ Returns a new point at the passed Z value

        Args:
            z(float): Elevation of new Points

        Returns:
            (:any:`XYZ`): New Points
        """
        return XYZ(self.x, self.y, z) if wrapped else DB.XYZ(self.x, self.y, z)

    @property
    def as_tuple(self):
        """
        Tuple representing the xyz coordinate of the Point

        Returns:
            (tuple): tuple float of XYZ values

        """
        return (self.x, self.y, self.z)

    @property
    def as_dict(self):
        """
        Dictionary representing the xyz coordinate of the Point

        Returns:
            (dict): dict with float of XYZ values

        """
        return OrderedDict([('x', self.x), ('y', self.y), ('z', self.z)])

    def rotate(self, rotation, axis=None, radians=False):
        rotated_xyz = Transform.rotate_vector(self.unwrap(),
                                              rotation,
                                              center=None,
                                              axis=axis,
                                              radians=radians)
        return rotated_xyz

    def __mul__(self, value):
        """ Multiplication Method """
        return XYZ(self.unwrap() * value)

    def __add__(self, point):
        """ Addition Method """
        return XYZ(self.unwrap() + XYZ(point).unwrap())

    def __sub__(self, point):
        """ Subtraction Method """
        return XYZ(self.unwrap() - XYZ(point).unwrap())

    def __eq__(self, other):
        """ Equality Method """
        return self._revit_object.IsAlmostEqualTo(XYZ(other).unwrap())

    def __repr__(self):
        return super(XYZ, self).__repr__(data=self.as_dict,
                                         to_string='Autodesk.Revit.DB.XYZ')



class HeaderedContentControl(ContentControl,IResource,IAnimatable,IInputElement,IFrameworkInputElement,ISupportInitialize,IHaveResources,IQueryAmbient,IAddChild):
 """
 Provides the base implementation for all controls that contain single content and have a header.

 

 HeaderedContentControl()
 """
 def AddChild(self,*args):
  """
  AddChild(self: ContentControl,value: object)

   Adds a specified object as the child of a System.Windows.Controls.ContentControl.

  

   value: The object to add.
  """
  pass
 def AddLogicalChild(self,*args):
  """
  AddLogicalChild(self: FrameworkElement,child: object)

   Adds the provided object to the logical tree of this element.

  

   child: Child element to be added.
  """
  pass
 def AddText(self,*args):
  """
  AddText(self: ContentControl,text: str)

   Adds a specified text string to a System.Windows.Controls.ContentControl.

  

   text: The string to add.
  """
  pass
 def AddVisualChild(self,*args):
  """
  AddVisualChild(self: Visual,child: Visual)

   Defines the parent-child relationship between two visuals.

  

   child: The child visual object to add to parent visual.
  """
  pass
 def ArrangeCore(self,*args):
  """
  ArrangeCore(self: FrameworkElement,finalRect: Rect)

   Implements System.Windows.UIElement.ArrangeCore(System.Windows.Rect) (defined as virtual in 

    System.Windows.UIElement) and seals the implementation.

  

  

   finalRect: The final area within the parent that this element should use to arrange itself and its children.
  """
  pass
 def ArrangeOverride(self,*args):
  """
  ArrangeOverride(self: Control,arrangeBounds: Size) -> Size

  

   Called to arrange and size the content of a System.Windows.Controls.Control object.

  

   arrangeBounds: The computed size that is used to arrange the content.

   Returns: The size of the control.
  """
  pass
 def GetLayoutClip(self,*args):
  """
  GetLayoutClip(self: FrameworkElement,layoutSlotSize: Size) -> Geometry

  

   Returns a geometry for a clipping mask. The mask applies if the layout system attempts to 

    arrange an element that is larger than the available display space.

  

  

   layoutSlotSize: The size of the part of the element that does visual presentation.

   Returns: The clipping geometry.
  """
  pass
 def GetTemplateChild(self,*args):
  """
  GetTemplateChild(self: FrameworkElement,childName: str) -> DependencyObject

  

   Returns the named element in the visual tree of an instantiated 

    System.Windows.Controls.ControlTemplate.

  

  

   childName: Name of the child to find.

   Returns: The requested element. May be null if no element of the requested name exists.
  """
  pass
 def GetUIParentCore(self,*args):
  """
  GetUIParentCore(self: FrameworkElement) -> DependencyObject

  

   Returns an alternative logical parent for this element if there is no visual parent.

   Returns: Returns something other than null whenever a WPF framework-level implementation of this method 

    has a non-visual parent connection.
  """
  pass
 def GetVisualChild(self,*args):
  """
  GetVisualChild(self: FrameworkElement,index: int) -> Visual

  

   Overrides System.Windows.Media.Visual.GetVisualChild(System.Int32),and returns a child at the 

    specified index from a collection of child elements.

  

  

   index: The zero-based index of the requested child element in the collection.

   Returns: The requested child element. This should not return null; if the provided index is out of range,

    an exception is thrown.
  """
  pass
 def HitTestCore(self,*args):
  """
  HitTestCore(self: UIElement,hitTestParameters: GeometryHitTestParameters) -> GeometryHitTestResult

  

   Implements 

    System.Windows.Media.Visual.HitTestCore(System.Windows.Media.GeometryHitTestParameters) to 

    supply base element hit testing behavior (returning System.Windows.Media.GeometryHitTestResult).

  

  

   hitTestParameters: Describes the hit test to perform,including the initial hit point.

   Returns: Results of the test,including the evaluated geometry.

  HitTestCore(self: UIElement,hitTestParameters: PointHitTestParameters) -> HitTestResult

  

   Implements System.Windows.Media.Visual.HitTestCore(System.Windows.Media.PointHitTestParameters) 

    to supply base element hit testing behavior (returning System.Windows.Media.HitTestResult).

  

  

   hitTestParameters: Describes the hit test to perform,including the initial hit point.

   Returns: Results of the test,including the evaluated point.
  """
  pass
 def MeasureCore(self,*args):
  """
  MeasureCore(self: FrameworkElement,availableSize: Size) -> Size

  

   Implements basic measure-pass layout system behavior for System.Windows.FrameworkElement.

  

   availableSize: The available size that the parent element can give to the child elements.

   Returns: The desired size of this element in layout.
  """
  pass
 def MeasureOverride(self,*args):
  """
  MeasureOverride(self: Control,constraint: Size) -> Size

  

   Called to remeasure a control.

  

   constraint: The maximum size that the method can return.

   Returns: The size of the control,up to the maximum specified by constraint.
  """
  pass
 def OnAccessKey(self,*args):
  """
  OnAccessKey(self: UIElement,e: AccessKeyEventArgs)

   Provides class handling for when an access key that is meaningful for this element is invoked.

  

   e: The event data to the access key event. The event data reports which key was invoked,and 

    indicate whether the System.Windows.Input.AccessKeyManager object that controls the sending of 

    these events also sent this access key invocation to other elements.
  """
  pass
 def OnChildDesiredSizeChanged(self,*args):
  """
  OnChildDesiredSizeChanged(self: UIElement,child: UIElement)

   Supports layout behavior when a child element is resized.

  

   child: The child element that is being resized.
  """
  pass
 def OnContentChanged(self,*args):
  """
  OnContentChanged(self: ContentControl,oldContent: object,newContent: object)

   Called when the System.Windows.Controls.ContentControl.Content property changes.

  

   oldContent: The old value of the System.Windows.Controls.ContentControl.Content property.

   newContent: The new value of the System.Windows.Controls.ContentControl.Content property.
  """
  pass
 def OnContentStringFormatChanged(self,*args):
  """
  OnContentStringFormatChanged(self: ContentControl,oldContentStringFormat: str,newContentStringFormat: str)

   Occurs when the System.Windows.Controls.ContentControl.ContentStringFormat property changes.

  

   oldContentStringFormat: The old value of System.Windows.Controls.ContentControl.ContentStringFormat.

   newContentStringFormat: The new value of System.Windows.Controls.ContentControl.ContentStringFormat.
  """
  pass
 def OnContentTemplateChanged(self,*args):
  """
  OnContentTemplateChanged(self: ContentControl,oldContentTemplate: DataTemplate,newContentTemplate: DataTemplate)

   Called when the System.Windows.Controls.ContentControl.ContentTemplate property changes.

  

   oldContentTemplate: The old value of the System.Windows.Controls.ContentControl.ContentTemplate property.

   newContentTemplate: The new value of the System.Windows.Controls.ContentControl.ContentTemplate property.
  """
  pass
 def OnContentTemplateSelectorChanged(self,*args):
  """
  OnContentTemplateSelectorChanged(self: ContentControl,oldContentTemplateSelector: DataTemplateSelector,newContentTemplateSelector: DataTemplateSelector)

   Called when the System.Windows.Controls.ContentControl.ContentTemplateSelector property changes.

  

   oldContentTemplateSelector: The old value of the System.Windows.Controls.ContentControl.ContentTemplateSelector property.

   newContentTemplateSelector: The new value of the System.Windows.Controls.ContentControl.ContentTemplateSelector property.
  """
  pass
 def OnContextMenuClosing(self,*args):
  """
  OnContextMenuClosing(self: FrameworkElement,e: ContextMenuEventArgs)

   Invoked whenever an unhandled System.Windows.FrameworkElement.ContextMenuClosing routed event 

    reaches this class in its route. Implement this method to add class handling for this event.

  

  

   e: Provides data about the event.
  """
  pass
 def OnContextMenuOpening(self,*args):
  """
  OnContextMenuOpening(self: FrameworkElement,e: ContextMenuEventArgs)

   Invoked whenever an unhandled System.Windows.FrameworkElement.ContextMenuOpening routed event 

    reaches this class in its route. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.RoutedEventArgs that contains the event data.
  """
  pass
 def OnCreateAutomationPeer(self,*args):
  """
  OnCreateAutomationPeer(self: UIElement) -> AutomationPeer

  

   Returns class-specific System.Windows.Automation.Peers.AutomationPeer implementations for the 

    Windows Presentation Foundation (WPF) infrastructure.

  

   Returns: The type-specific System.Windows.Automation.Peers.AutomationPeer implementation.
  """
  pass
 def OnDpiChanged(self,*args):
  """ OnDpiChanged(self: Visual,oldDpi: DpiScale,newDpi: DpiScale) """
  pass
 def OnDragEnter(self,*args):
  """
  OnDragEnter(self: UIElement,e: DragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.DragEnter�attached event reaches an element in 

    its route that is derived from this class. Implement this method to add class handling for this 

    event.

  

  

   e: The System.Windows.DragEventArgs that contains the event data.
  """
  pass
 def OnDragLeave(self,*args):
  """
  OnDragLeave(self: UIElement,e: DragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.DragLeave�attached event reaches an element in 

    its route that is derived from this class. Implement this method to add class handling for this 

    event.

  

  

   e: The System.Windows.DragEventArgs that contains the event data.
  """
  pass
 def OnDragOver(self,*args):
  """
  OnDragOver(self: UIElement,e: DragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.DragOver�attached event reaches an element in 

    its route that is derived from this class. Implement this method to add class handling for this 

    event.

  

  

   e: The System.Windows.DragEventArgs that contains the event data.
  """
  pass
 def OnDrop(self,*args):
  """
  OnDrop(self: UIElement,e: DragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.DragEnter�attached event reaches an element in 

    its route that is derived from this class. Implement this method to add class handling for this 

    event.

  

  

   e: The System.Windows.DragEventArgs that contains the event data.
  """
  pass
 def OnGiveFeedback(self,*args):
  """
  OnGiveFeedback(self: UIElement,e: GiveFeedbackEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.GiveFeedback�attached event reaches an element 

    in its route that is derived from this class. Implement this method to add class handling for 

    this event.

  

  

   e: The System.Windows.GiveFeedbackEventArgs that contains the event data.
  """
  pass
 def OnGotFocus(self,*args):
  """
  OnGotFocus(self: FrameworkElement,e: RoutedEventArgs)

   Invoked whenever an unhandled System.Windows.UIElement.GotFocus event reaches this element in 

    its route.

  

  

   e: The System.Windows.RoutedEventArgs that contains the event data.
  """
  pass
 def OnGotKeyboardFocus(self,*args):
  """
  OnGotKeyboardFocus(self: UIElement,e: KeyboardFocusChangedEventArgs)

   Invoked when an unhandled System.Windows.Input.Keyboard.GotKeyboardFocus�attached event reaches 

    an element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.KeyboardFocusChangedEventArgs that contains the event data.
  """
  pass
 def OnGotMouseCapture(self,*args):
  """
  OnGotMouseCapture(self: UIElement,e: MouseEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.GotMouseCapture�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseEventArgs that contains the event data.
  """
  pass
 def OnGotStylusCapture(self,*args):
  """
  OnGotStylusCapture(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.GotStylusCapture�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnGotTouchCapture(self,*args):
  """
  OnGotTouchCapture(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.GotTouchCapture routed event that 

    occurs when a touch is captured to this element.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnHeaderChanged(self,*args):
  """
  OnHeaderChanged(self: HeaderedContentControl,oldHeader: object,newHeader: object)

   Called when the System.Windows.Controls.HeaderedContentControl.Header property of a 

    System.Windows.Controls.HeaderedContentControl changes.

  

  

   oldHeader: Old value of the System.Windows.Controls.HeaderedContentControl.Header property.

   newHeader: New value of the System.Windows.Controls.HeaderedContentControl.Header property.
  """
  pass
 def OnHeaderStringFormatChanged(self,*args):
  """
  OnHeaderStringFormatChanged(self: HeaderedContentControl,oldHeaderStringFormat: str,newHeaderStringFormat: str)

   Called when the System.Windows.Controls.HeaderedContentControl.HeaderStringFormat property 

    changes.

  

  

   oldHeaderStringFormat: The old value of the System.Windows.Controls.HeaderedContentControl.HeaderStringFormat property.

   newHeaderStringFormat: The new value of the System.Windows.Controls.HeaderedContentControl.HeaderStringFormat property.
  """
  pass
 def OnHeaderTemplateChanged(self,*args):
  """
  OnHeaderTemplateChanged(self: HeaderedContentControl,oldHeaderTemplate: DataTemplate,newHeaderTemplate: DataTemplate)

   Called when the System.Windows.Controls.HeaderedContentControl.HeaderTemplate property changes.

  

   oldHeaderTemplate: Old value of the System.Windows.Controls.HeaderedContentControl.HeaderTemplate property.

   newHeaderTemplate: New value of the System.Windows.Controls.HeaderedContentControl.HeaderTemplate property.
  """
  pass
 def OnHeaderTemplateSelectorChanged(self,*args):
  """
  OnHeaderTemplateSelectorChanged(self: HeaderedContentControl,oldHeaderTemplateSelector: DataTemplateSelector,newHeaderTemplateSelector: DataTemplateSelector)

   Called when the System.Windows.Controls.HeaderedContentControl.HeaderTemplateSelector property 

    changes.

  

  

   oldHeaderTemplateSelector: Old value of the System.Windows.Controls.HeaderedContentControl.HeaderTemplateSelector property.

   newHeaderTemplateSelector: New value of the System.Windows.Controls.HeaderedContentControl.HeaderTemplateSelector property.
  """
  pass
 def OnInitialized(self,*args):
  """
  OnInitialized(self: FrameworkElement,e: EventArgs)

   Raises the System.Windows.FrameworkElement.Initialized event. This method is invoked whenever 

    System.Windows.FrameworkElement.IsInitialized is set to true internally.

  

  

   e: The System.Windows.RoutedEventArgs that contains the event data.
  """
  pass
 def OnIsKeyboardFocusedChanged(self,*args):
  """
  OnIsKeyboardFocusedChanged(self: UIElement,e: DependencyPropertyChangedEventArgs)

   Invoked when an unhandled System.Windows.UIElement.IsKeyboardFocusedChanged event is raised on 

    this element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.DependencyPropertyChangedEventArgs that contains the event data.
  """
  pass
 def OnIsKeyboardFocusWithinChanged(self,*args):
  """
  OnIsKeyboardFocusWithinChanged(self: UIElement,e: DependencyPropertyChangedEventArgs)

   Invoked just before the System.Windows.UIElement.IsKeyboardFocusWithinChanged event is raised by 

    this element. Implement this method to add class handling for this event.

  

  

   e: A System.Windows.DependencyPropertyChangedEventArgs that contains the event data.
  """
  pass
 def OnIsMouseCapturedChanged(self,*args):
  """
  OnIsMouseCapturedChanged(self: UIElement,e: DependencyPropertyChangedEventArgs)

   Invoked when an unhandled System.Windows.UIElement.IsMouseCapturedChanged event is raised on 

    this element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.DependencyPropertyChangedEventArgs that contains the event data.
  """
  pass
 def OnIsMouseCaptureWithinChanged(self,*args):
  """
  OnIsMouseCaptureWithinChanged(self: UIElement,e: DependencyPropertyChangedEventArgs)

   Invoked when an unhandled System.Windows.UIElement.IsMouseCaptureWithinChanged event is raised 

    on this element. Implement this method to add class handling for this event.

  

  

   e: A System.Windows.DependencyPropertyChangedEventArgs that contains the event data.
  """
  pass
 def OnIsMouseDirectlyOverChanged(self,*args):
  """
  OnIsMouseDirectlyOverChanged(self: UIElement,e: DependencyPropertyChangedEventArgs)

   Invoked when an unhandled System.Windows.UIElement.IsMouseDirectlyOverChanged event is raised on 

    this element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.DependencyPropertyChangedEventArgs that contains the event data.
  """
  pass
 def OnIsStylusCapturedChanged(self,*args):
  """
  OnIsStylusCapturedChanged(self: UIElement,e: DependencyPropertyChangedEventArgs)

   Invoked when an unhandled System.Windows.UIElement.IsStylusCapturedChanged event is raised on 

    this element. Implement this method to add class handling for this event.

  

  

   e: A System.Windows.DependencyPropertyChangedEventArgs that contains the event data.
  """
  pass
 def OnIsStylusCaptureWithinChanged(self,*args):
  """
  OnIsStylusCaptureWithinChanged(self: UIElement,e: DependencyPropertyChangedEventArgs)

   Invoked when an unhandled System.Windows.UIElement.IsStylusCaptureWithinChanged event is raised 

    on this element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.DependencyPropertyChangedEventArgs that contains the event data.
  """
  pass
 def OnIsStylusDirectlyOverChanged(self,*args):
  """
  OnIsStylusDirectlyOverChanged(self: UIElement,e: DependencyPropertyChangedEventArgs)

   Invoked when an unhandled System.Windows.UIElement.IsStylusDirectlyOverChanged event is raised 

    on this element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.DependencyPropertyChangedEventArgs that contains the event data.
  """
  pass
 def OnKeyDown(self,*args):
  """
  OnKeyDown(self: UIElement,e: KeyEventArgs)

   Invoked when an unhandled System.Windows.Input.Keyboard.KeyDown�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.KeyEventArgs that contains the event data.
  """
  pass
 def OnKeyUp(self,*args):
  """
  OnKeyUp(self: UIElement,e: KeyEventArgs)

   Invoked when an unhandled System.Windows.Input.Keyboard.KeyUp�attached event reaches an element 

    in its route that is derived from this class. Implement this method to add class handling for 

    this event.

  

  

   e: The System.Windows.Input.KeyEventArgs that contains the event data.
  """
  pass
 def OnLostFocus(self,*args):
  """
  OnLostFocus(self: UIElement,e: RoutedEventArgs)

   Raises the System.Windows.UIElement.LostFocus�routed event by using the event data that is 

    provided.

  

  

   e: A System.Windows.RoutedEventArgs that contains event data. This event data must contain the 

    identifier for the System.Windows.UIElement.LostFocus event.
  """
  pass
 def OnLostKeyboardFocus(self,*args):
  """
  OnLostKeyboardFocus(self: UIElement,e: KeyboardFocusChangedEventArgs)

   Invoked when an unhandled System.Windows.Input.Keyboard.LostKeyboardFocus�attached event reaches 

    an element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.KeyboardFocusChangedEventArgs that contains event data.
  """
  pass
 def OnLostMouseCapture(self,*args):
  """
  OnLostMouseCapture(self: UIElement,e: MouseEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.LostMouseCapture�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseEventArgs that contains event data.
  """
  pass
 def OnLostStylusCapture(self,*args):
  """
  OnLostStylusCapture(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.LostStylusCapture�attached event reaches 

    an element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains event data.
  """
  pass
 def OnLostTouchCapture(self,*args):
  """
  OnLostTouchCapture(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.LostTouchCapture routed event that 

    occurs when this element loses a touch capture.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnManipulationBoundaryFeedback(self,*args):
  """
  OnManipulationBoundaryFeedback(self: UIElement,e: ManipulationBoundaryFeedbackEventArgs)

   Called when the System.Windows.UIElement.ManipulationBoundaryFeedback event occurs.

  

   e: The data for the event.
  """
  pass
 def OnManipulationCompleted(self,*args):
  """
  OnManipulationCompleted(self: UIElement,e: ManipulationCompletedEventArgs)

   Called when the System.Windows.UIElement.ManipulationCompleted event occurs.

  

   e: The data for the event.
  """
  pass
 def OnManipulationDelta(self,*args):
  """
  OnManipulationDelta(self: UIElement,e: ManipulationDeltaEventArgs)

   Called when the System.Windows.UIElement.ManipulationDelta event occurs.

  

   e: The data for the event.
  """
  pass
 def OnManipulationInertiaStarting(self,*args):
  """
  OnManipulationInertiaStarting(self: UIElement,e: ManipulationInertiaStartingEventArgs)

   Called when the System.Windows.UIElement.ManipulationInertiaStarting event occurs.

  

   e: The data for the event.
  """
  pass
 def OnManipulationStarted(self,*args):
  """
  OnManipulationStarted(self: UIElement,e: ManipulationStartedEventArgs)

   Called when the System.Windows.UIElement.ManipulationStarted event occurs.

  

   e: The data for the event.
  """
  pass
 def OnManipulationStarting(self,*args):
  """
  OnManipulationStarting(self: UIElement,e: ManipulationStartingEventArgs)

   Provides class handling for the System.Windows.UIElement.ManipulationStarting routed event that 

    occurs when the manipulation processor is first created.

  

  

   e: A System.Windows.Input.ManipulationStartingEventArgs  that contains the event data.
  """
  pass
 def OnMouseDoubleClick(self,*args):
  """
  OnMouseDoubleClick(self: Control,e: MouseButtonEventArgs)

   Raises the System.Windows.Controls.Control.MouseDoubleClick routed event.

  

   e: The event data.
  """
  pass
 def OnMouseDown(self,*args):
  """
  OnMouseDown(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.MouseDown�attached event reaches an element 

    in its route that is derived from this class. Implement this method to add class handling for 

    this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. This event data 

    reports details about the mouse button that was pressed and the handled state.
  """
  pass
 def OnMouseEnter(self,*args):
  """
  OnMouseEnter(self: UIElement,e: MouseEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.MouseEnter�attached event is raised on this 

    element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.Input.MouseEventArgs that contains the event data.
  """
  pass
 def OnMouseLeave(self,*args):
  """
  OnMouseLeave(self: UIElement,e: MouseEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.MouseLeave�attached event is raised on this 

    element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.Input.MouseEventArgs that contains the event data.
  """
  pass
 def OnMouseLeftButtonDown(self,*args):
  """
  OnMouseLeftButtonDown(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.UIElement.MouseLeftButtonDown�routed event is raised on 

    this element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the left mouse button was pressed.
  """
  pass
 def OnMouseLeftButtonUp(self,*args):
  """
  OnMouseLeftButtonUp(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.UIElement.MouseLeftButtonUp�routed event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the left mouse button was released.
  """
  pass
 def OnMouseMove(self,*args):
  """
  OnMouseMove(self: UIElement,e: MouseEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.MouseMove�attached event reaches an element 

    in its route that is derived from this class. Implement this method to add class handling for 

    this event.

  

  

   e: The System.Windows.Input.MouseEventArgs that contains the event data.
  """
  pass
 def OnMouseRightButtonDown(self,*args):
  """
  OnMouseRightButtonDown(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.UIElement.MouseRightButtonDown�routed event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the right mouse button was pressed.
  """
  pass
 def OnMouseRightButtonUp(self,*args):
  """
  OnMouseRightButtonUp(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.UIElement.MouseRightButtonUp�routed event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the right mouse button was released.
  """
  pass
 def OnMouseUp(self,*args):
  """
  OnMouseUp(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.MouseUp�routed event reaches an element in 

    its route that is derived from this class. Implement this method to add class handling for this 

    event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the mouse button was released.
  """
  pass
 def OnMouseWheel(self,*args):
  """
  OnMouseWheel(self: UIElement,e: MouseWheelEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.MouseWheel�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseWheelEventArgs that contains the event data.
  """
  pass
 def OnPreviewDragEnter(self,*args):
  """
  OnPreviewDragEnter(self: UIElement,e: DragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.PreviewDragEnter�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.DragEventArgs that contains the event data.
  """
  pass
 def OnPreviewDragLeave(self,*args):
  """
  OnPreviewDragLeave(self: UIElement,e: DragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.PreviewDragLeave�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.DragEventArgs that contains the event data.
  """
  pass
 def OnPreviewDragOver(self,*args):
  """
  OnPreviewDragOver(self: UIElement,e: DragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.PreviewDragOver�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.DragEventArgs that contains the event data.
  """
  pass
 def OnPreviewDrop(self,*args):
  """
  OnPreviewDrop(self: UIElement,e: DragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.PreviewDrop�attached event reaches an element 

    in its route that is derived from this class. Implement this method to add class handling for 

    this event.

  

  

   e: The System.Windows.DragEventArgs that contains the event data.
  """
  pass
 def OnPreviewGiveFeedback(self,*args):
  """
  OnPreviewGiveFeedback(self: UIElement,e: GiveFeedbackEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.PreviewGiveFeedback�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.GiveFeedbackEventArgs that contains the event data.
  """
  pass
 def OnPreviewGotKeyboardFocus(self,*args):
  """
  OnPreviewGotKeyboardFocus(self: UIElement,e: KeyboardFocusChangedEventArgs)

   Invoked when an unhandled System.Windows.Input.Keyboard.PreviewGotKeyboardFocus�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.KeyboardFocusChangedEventArgs that contains the event data.
  """
  pass
 def OnPreviewKeyDown(self,*args):
  """
  OnPreviewKeyDown(self: UIElement,e: KeyEventArgs)

   Invoked when an unhandled System.Windows.Input.Keyboard.PreviewKeyDown�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.KeyEventArgs that contains the event data.
  """
  pass
 def OnPreviewKeyUp(self,*args):
  """
  OnPreviewKeyUp(self: UIElement,e: KeyEventArgs)

   Invoked when an unhandled System.Windows.Input.Keyboard.PreviewKeyUp�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.KeyEventArgs that contains the event data.
  """
  pass
 def OnPreviewLostKeyboardFocus(self,*args):
  """
  OnPreviewLostKeyboardFocus(self: UIElement,e: KeyboardFocusChangedEventArgs)

   Invoked when an unhandled System.Windows.Input.Keyboard.PreviewKeyDown�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.KeyboardFocusChangedEventArgs that contains the event data.
  """
  pass
 def OnPreviewMouseDoubleClick(self,*args):
  """
  OnPreviewMouseDoubleClick(self: Control,e: MouseButtonEventArgs)

   Raises the System.Windows.Controls.Control.PreviewMouseDoubleClick routed event.

  

   e: The event data.
  """
  pass
 def OnPreviewMouseDown(self,*args):
  """
  OnPreviewMouseDown(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.PreviewMouseDown attached�routed event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that one or more mouse buttons were pressed.
  """
  pass
 def OnPreviewMouseLeftButtonDown(self,*args):
  """
  OnPreviewMouseLeftButtonDown(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.UIElement.PreviewMouseLeftButtonDown�routed event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the left mouse button was pressed.
  """
  pass
 def OnPreviewMouseLeftButtonUp(self,*args):
  """
  OnPreviewMouseLeftButtonUp(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.UIElement.PreviewMouseLeftButtonUp�routed event reaches 

    an element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the left mouse button was released.
  """
  pass
 def OnPreviewMouseMove(self,*args):
  """
  OnPreviewMouseMove(self: UIElement,e: MouseEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.PreviewMouseMove�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseEventArgs that contains the event data.
  """
  pass
 def OnPreviewMouseRightButtonDown(self,*args):
  """
  OnPreviewMouseRightButtonDown(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.UIElement.PreviewMouseRightButtonDown�routed event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the right mouse button was pressed.
  """
  pass
 def OnPreviewMouseRightButtonUp(self,*args):
  """
  OnPreviewMouseRightButtonUp(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.UIElement.PreviewMouseRightButtonUp�routed event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that the right mouse button was released.
  """
  pass
 def OnPreviewMouseUp(self,*args):
  """
  OnPreviewMouseUp(self: UIElement,e: MouseButtonEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.PreviewMouseUp�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseButtonEventArgs that contains the event data. The event data 

    reports that one or more mouse buttons were released.
  """
  pass
 def OnPreviewMouseWheel(self,*args):
  """
  OnPreviewMouseWheel(self: UIElement,e: MouseWheelEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.PreviewMouseWheel�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.MouseWheelEventArgs that contains the event data.
  """
  pass
 def OnPreviewQueryContinueDrag(self,*args):
  """
  OnPreviewQueryContinueDrag(self: UIElement,e: QueryContinueDragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.PreviewQueryContinueDrag�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.QueryContinueDragEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusButtonDown(self,*args):
  """
  OnPreviewStylusButtonDown(self: UIElement,e: StylusButtonEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusButtonDown�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.StylusButtonEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusButtonUp(self,*args):
  """
  OnPreviewStylusButtonUp(self: UIElement,e: StylusButtonEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusButtonUp�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.StylusButtonEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusDown(self,*args):
  """
  OnPreviewStylusDown(self: UIElement,e: StylusDownEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusDown�attached event reaches 

    an element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusDownEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusInAirMove(self,*args):
  """
  OnPreviewStylusInAirMove(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusInAirMove�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusInRange(self,*args):
  """
  OnPreviewStylusInRange(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusInRange�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusMove(self,*args):
  """
  OnPreviewStylusMove(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusMove�attached event reaches 

    an element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusOutOfRange(self,*args):
  """
  OnPreviewStylusOutOfRange(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusOutOfRange�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusSystemGesture(self,*args):
  """
  OnPreviewStylusSystemGesture(self: UIElement,e: StylusSystemGestureEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusSystemGesture�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.StylusSystemGestureEventArgs that contains the event data.
  """
  pass
 def OnPreviewStylusUp(self,*args):
  """
  OnPreviewStylusUp(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.PreviewStylusUp�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnPreviewTextInput(self,*args):
  """
  OnPreviewTextInput(self: UIElement,e: TextCompositionEventArgs)

   Invoked when an unhandled System.Windows.Input.TextCompositionManager.PreviewTextInput�attached 

    event reaches an element in its route that is derived from this class. Implement this method to 

    add class handling for this event.

  

  

   e: The System.Windows.Input.TextCompositionEventArgs that contains the event data.
  """
  pass
 def OnPreviewTouchDown(self,*args):
  """
  OnPreviewTouchDown(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.PreviewTouchDown routed event that 

    occurs when a touch presses this element.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnPreviewTouchMove(self,*args):
  """
  OnPreviewTouchMove(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.PreviewTouchMove routed event that 

    occurs when a touch moves while inside this element.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnPreviewTouchUp(self,*args):
  """
  OnPreviewTouchUp(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.PreviewTouchUp routed event that occurs 

    when a touch is released inside this element.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnPropertyChanged(self,*args):
  """
  OnPropertyChanged(self: FrameworkElement,e: DependencyPropertyChangedEventArgs)

   Invoked whenever the effective value of any dependency property on this 

    System.Windows.FrameworkElement has been updated. The specific dependency property that changed 

    is reported in the arguments parameter. Overrides 

    System.Windows.DependencyObject.OnPropertyChanged(System.Windows.DependencyPropertyChangedEventAr

    gs).

  

  

   e: The event data that describes the property that changed,as well as old and new values.
  """
  pass
 def OnQueryContinueDrag(self,*args):
  """
  OnQueryContinueDrag(self: UIElement,e: QueryContinueDragEventArgs)

   Invoked when an unhandled System.Windows.DragDrop.QueryContinueDrag�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.QueryContinueDragEventArgs that contains the event data.
  """
  pass
 def OnQueryCursor(self,*args):
  """
  OnQueryCursor(self: UIElement,e: QueryCursorEventArgs)

   Invoked when an unhandled System.Windows.Input.Mouse.QueryCursor�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.QueryCursorEventArgs that contains the event data.
  """
  pass
 def OnRender(self,*args):
  """
  OnRender(self: UIElement,drawingContext: DrawingContext)

   When overridden in a derived class,participates in rendering operations that are directed by 

    the layout system. The rendering instructions for this element are not used directly when this 

    method is invoked,and are instead preserved for later asynchronous use by layout and drawing.

  

  

   drawingContext: The drawing instructions for a specific element. This context is provided to the layout system.
  """
  pass
 def OnRenderSizeChanged(self,*args):
  """
  OnRenderSizeChanged(self: FrameworkElement,sizeInfo: SizeChangedInfo)

   Raises the System.Windows.FrameworkElement.SizeChanged event,using the specified information as 

    part of the eventual event data.

  

  

   sizeInfo: Details of the old and new size involved in the change.
  """
  pass
 def OnStyleChanged(self,*args):
  """
  OnStyleChanged(self: FrameworkElement,oldStyle: Style,newStyle: Style)

   Invoked when the style in use on this element changes,which will invalidate the layout.

  

   oldStyle: The old style.

   newStyle: The new style.
  """
  pass
 def OnStylusButtonDown(self,*args):
  """
  OnStylusButtonDown(self: UIElement,e: StylusButtonEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusButtonDown�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusButtonEventArgs that contains the event data.
  """
  pass
 def OnStylusButtonUp(self,*args):
  """
  OnStylusButtonUp(self: UIElement,e: StylusButtonEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusButtonUp�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusButtonEventArgs that contains the event data.
  """
  pass
 def OnStylusDown(self,*args):
  """
  OnStylusDown(self: UIElement,e: StylusDownEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusDown�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusDownEventArgs that contains the event data.
  """
  pass
 def OnStylusEnter(self,*args):
  """
  OnStylusEnter(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusEnter�attached event is raised by 

    this element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnStylusInAirMove(self,*args):
  """
  OnStylusInAirMove(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusInAirMove�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnStylusInRange(self,*args):
  """
  OnStylusInRange(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusInRange�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnStylusLeave(self,*args):
  """
  OnStylusLeave(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusLeave�attached event is raised by 

    this element. Implement this method to add class handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnStylusMove(self,*args):
  """
  OnStylusMove(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusMove�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnStylusOutOfRange(self,*args):
  """
  OnStylusOutOfRange(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusOutOfRange�attached event reaches an 

    element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnStylusSystemGesture(self,*args):
  """
  OnStylusSystemGesture(self: UIElement,e: StylusSystemGestureEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusSystemGesture�attached event reaches 

    an element in its route that is derived from this class. Implement this method to add class 

    handling for this event.

  

  

   e: The System.Windows.Input.StylusSystemGestureEventArgs that contains the event data.
  """
  pass
 def OnStylusUp(self,*args):
  """
  OnStylusUp(self: UIElement,e: StylusEventArgs)

   Invoked when an unhandled System.Windows.Input.Stylus.StylusUp�attached event reaches an element 

    in its route that is derived from this class. Implement this method to add class handling for 

    this event.

  

  

   e: The System.Windows.Input.StylusEventArgs that contains the event data.
  """
  pass
 def OnTemplateChanged(self,*args):
  """
  OnTemplateChanged(self: Control,oldTemplate: ControlTemplate,newTemplate: ControlTemplate)

   Called whenever the control's template changes.

  

   oldTemplate: The old template.

   newTemplate: The new template.
  """
  pass
 def OnTextInput(self,*args):
  """
  OnTextInput(self: UIElement,e: TextCompositionEventArgs)

   Invoked when an unhandled System.Windows.Input.TextCompositionManager.TextInput�attached event 

    reaches an element in its route that is derived from this class. Implement this method to add 

    class handling for this event.

  

  

   e: The System.Windows.Input.TextCompositionEventArgs that contains the event data.
  """
  pass
 def OnToolTipClosing(self,*args):
  """
  OnToolTipClosing(self: FrameworkElement,e: ToolTipEventArgs)

   Invoked whenever an unhandled System.Windows.FrameworkElement.ToolTipClosing routed event 

    reaches this class in its route. Implement this method to add class handling for this event.

  

  

   e: Provides data about the event.
  """
  pass
 def OnToolTipOpening(self,*args):
  """
  OnToolTipOpening(self: FrameworkElement,e: ToolTipEventArgs)

   Invoked whenever the System.Windows.FrameworkElement.ToolTipOpening routed event reaches this 

    class in its route. Implement this method to add class handling for this event.

  

  

   e: Provides data about the event.
  """
  pass
 def OnTouchDown(self,*args):
  """
  OnTouchDown(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.TouchDown routed event that occurs when 

    a touch presses inside this element.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnTouchEnter(self,*args):
  """
  OnTouchEnter(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.TouchEnter routed event that occurs 

    when a touch moves from outside to inside the bounds of this element.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnTouchLeave(self,*args):
  """
  OnTouchLeave(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.TouchLeave routed event that occurs 

    when a touch moves from inside to outside the bounds of this System.Windows.UIElement.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnTouchMove(self,*args):
  """
  OnTouchMove(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.TouchMove routed event that occurs when 

    a touch moves while inside this element.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnTouchUp(self,*args):
  """
  OnTouchUp(self: UIElement,e: TouchEventArgs)

   Provides class handling for the System.Windows.UIElement.TouchUp routed event that occurs when a 

    touch is released inside this element.

  

  

   e: A System.Windows.Input.TouchEventArgs that contains the event data.
  """
  pass
 def OnVisualChildrenChanged(self,*args):
  """
  OnVisualChildrenChanged(self: Visual,visualAdded: DependencyObject,visualRemoved: DependencyObject)

   Called when the System.Windows.Media.VisualCollection of the visual object is modified.

  

   visualAdded: The System.Windows.Media.Visual that was added to the collection

   visualRemoved: The System.Windows.Media.Visual that was removed from the collection
  """
  pass
 def OnVisualParentChanged(self,*args):
  """
  OnVisualParentChanged(self: FrameworkElement,oldParent: DependencyObject)

   Invoked when the parent of this element in the visual tree is changed. Overrides 

    System.Windows.UIElement.OnVisualParentChanged(System.Windows.DependencyObject).

  

  

   oldParent: The old parent element. May be null to indicate that the element did not have a visual parent 

    previously.
  """
  pass
 def ParentLayoutInvalidated(self,*args):
  """
  ParentLayoutInvalidated(self: FrameworkElement,child: UIElement)

   Supports incremental layout implementations in specialized subclasses of 

    System.Windows.FrameworkElement. 

    System.Windows.FrameworkElement.ParentLayoutInvalidated(System.Windows.UIElement)  is invoked 

    when a child element has invalidated a property that is marked in metadata as affecting the 

    parent's measure or arrange passes during layout.

  

  

   child: The child element reporting the change.
  """
  pass
 def RemoveLogicalChild(self,*args):
  """
  RemoveLogicalChild(self: FrameworkElement,child: object)

   Removes the provided object from this element's logical tree. System.Windows.FrameworkElement 

    updates the affected logical tree parent pointers to keep in sync with this deletion.

  

  

   child: The element to remove.
  """
  pass
 def RemoveVisualChild(self,*args):
  """
  RemoveVisualChild(self: Visual,child: Visual)

   Removes the parent-child relationship between two visuals.

  

   child: The child visual object to remove from the parent visual.
  """
  pass
 def ShouldSerializeProperty(self,*args):
  """
  ShouldSerializeProperty(self: DependencyObject,dp: DependencyProperty) -> bool

  

   Returns a value that indicates whether serialization processes should serialize the value for 

    the provided dependency property.

  

  

   dp: The identifier for the dependency property that should be serialized.

   Returns: true if the dependency property that is supplied should be value-serialized; otherwise,false.
  """
  pass
 def ToString(self):
  """
  ToString(self: HeaderedContentControl) -> str

  

   Provides a string representation of a System.Windows.Controls.HeaderedContentControl.

   Returns: A string representation of the object.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass
 DefaultStyleKey=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the key to use to reference the style for this control,when theme styles are used or defined.



"""

 HandlesScrolling=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether a control supports scrolling.



"""

 HasEffectiveKeyboardFocus=property(lambda self: object(),lambda self,v: None,lambda self: None)

 HasHeader=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the header is null.



Get: HasHeader(self: HeaderedContentControl) -> bool



"""

 Header=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the data used for the header of each control.



Get: Header(self: HeaderedContentControl) -> object



Set: Header(self: HeaderedContentControl)=value

"""

 HeaderStringFormat=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a composite string that specifies how to format the System.Windows.Controls.HeaderedContentControl.Header property if it is displayed as a string.



Get: HeaderStringFormat(self: HeaderedContentControl) -> str



Set: HeaderStringFormat(self: HeaderedContentControl)=value

"""

 HeaderTemplate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the template used to display the content of the control's header.



Get: HeaderTemplate(self: HeaderedContentControl) -> DataTemplate



Set: HeaderTemplate(self: HeaderedContentControl)=value

"""

 HeaderTemplateSelector=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a data template selector that provides custom logic for choosing the template used to display the header.



Get: HeaderTemplateSelector(self: HeaderedContentControl) -> DataTemplateSelector



Set: HeaderTemplateSelector(self: HeaderedContentControl)=value

"""

 InheritanceBehavior=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the scope limits for property value inheritance,resource key lookup,and RelativeSource FindAncestor lookup.



"""

 IsEnabledCore=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that becomes the return value of System.Windows.UIElement.IsEnabled in derived classes.



"""

 LogicalChildren=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets an enumerator to the logical child elements of the System.Windows.Controls.ControlTemplate.



"""

 StylusPlugIns=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a collection of all stylus plug-in (customization) objects associated with this element.



"""

 VisualBitmapEffect=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.Effects.BitmapEffect value for the System.Windows.Media.Visual.



"""

 VisualBitmapEffectInput=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.Effects.BitmapEffectInput value for the System.Windows.Media.Visual.



"""

 VisualBitmapScalingMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.BitmapScalingMode for the System.Windows.Media.Visual.



"""

 VisualCacheMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a cached representation of the System.Windows.Media.Visual.



"""

 VisualChildrenCount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of visual child elements within this element.



"""

 VisualClearTypeHint=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.ClearTypeHint that determines how ClearType is rendered in the System.Windows.Media.Visual.



"""

 VisualClip=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the clip region of the System.Windows.Media.Visual as a System.Windows.Media.Geometry value.



"""

 VisualEdgeMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the edge mode of the System.Windows.Media.Visual as an System.Windows.Media.EdgeMode value.



"""

 VisualEffect=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the bitmap effect to apply to the System.Windows.Media.Visual.



"""

 VisualOffset=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the offset value of the visual object.



"""

 VisualOpacity=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the opacity of the System.Windows.Media.Visual.



"""

 VisualOpacityMask=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.Brush value that represents the opacity mask of the System.Windows.Media.Visual.



"""

 VisualParent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the visual tree parent of the visual object.



"""

 VisualScrollableAreaClip=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a clipped scrollable area for the System.Windows.Media.Visual.



"""

 VisualTextHintingMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.TextHintingMode of the System.Windows.Media.Visual.



"""

 VisualTextRenderingMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.TextRenderingMode of the System.Windows.Media.Visual.



"""

 VisualTransform=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Media.Transform value for the System.Windows.Media.Visual.



"""

 VisualXSnappingGuidelines=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the x-coordinate (vertical) guideline collection.



"""

 VisualYSnappingGuidelines=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the y-coordinate (horizontal) guideline collection.



"""


 HasHeaderProperty=None
 HeaderProperty=None
 HeaderStringFormatProperty=None
 HeaderTemplateProperty=None
 HeaderTemplateSelectorProperty=None




class UploadProgressChangedEventArgs(ProgressChangedEventArgs):
 """ Provides data for the System.Net.WebClient.UploadProgressChanged event of a System.Net.WebClient. """
 BytesReceived=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of bytes received.



Get: BytesReceived(self: UploadProgressChangedEventArgs) -> Int64



"""

 BytesSent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of bytes sent.



Get: BytesSent(self: UploadProgressChangedEventArgs) -> Int64



"""

 TotalBytesToReceive=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the total number of bytes in a System.Net.WebClient data upload operation.



Get: TotalBytesToReceive(self: UploadProgressChangedEventArgs) -> Int64



"""

 TotalBytesToSend=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the total number of bytes to send.



Get: TotalBytesToSend(self: UploadProgressChangedEventArgs) -> Int64



"""





# encoding: utf-8
# module System.Net.Mime calls itself Mime
# from System,Version=4.0.0.0,Culture=neutral,PublicKeyToken=b77a5c561934e089
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class ContentDisposition(object):
 """
 Represents a MIME protocol Content-Disposition header.

 

 ContentDisposition()

 ContentDisposition(disposition: str)
 """
 def Equals(self,rparam):
  """
  Equals(self: ContentDisposition,rparam: object) -> bool

  

   Determines whether the content-disposition header of the specified 

    System.Net.Mime.ContentDisposition object is equal to the content-disposition header of this 

    object.

  

  

   rparam: The System.Net.Mime.ContentDisposition object to compare with this object.

   Returns: true if the content-disposition headers are the same; otherwise false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: ContentDisposition) -> int

  

   Determines the hash code of the specified System.Net.Mime.ContentDisposition object

   Returns: An integer hash value.
  """
  pass
 def ToString(self):
  """
  ToString(self: ContentDisposition) -> str

  

   Returns a System.String representation of this instance.

   Returns: A System.String that contains the property values for this instance.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,disposition=None):
  """
  __new__(cls: type)

  __new__(cls: type,disposition: str)
  """
  pass
 def __ne__(self,*args):
  pass
 CreationDate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the creation date for a file attachment.



Get: CreationDate(self: ContentDisposition) -> DateTime



Set: CreationDate(self: ContentDisposition)=value

"""

 DispositionType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the disposition type for an e-mail attachment.



Get: DispositionType(self: ContentDisposition) -> str



Set: DispositionType(self: ContentDisposition)=value

"""

 FileName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the suggested file name for an e-mail attachment.



Get: FileName(self: ContentDisposition) -> str



Set: FileName(self: ContentDisposition)=value

"""

 Inline=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a System.Boolean value that determines the disposition type (Inline or Attachment) for an e-mail attachment.



Get: Inline(self: ContentDisposition) -> bool



Set: Inline(self: ContentDisposition)=value

"""

 ModificationDate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the modification date for a file attachment.



Get: ModificationDate(self: ContentDisposition) -> DateTime



Set: ModificationDate(self: ContentDisposition)=value

"""

 Parameters=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the parameters included in the Content-Disposition header represented by this instance.



Get: Parameters(self: ContentDisposition) -> StringDictionary



"""

 ReadDate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the read date for a file attachment.



Get: ReadDate(self: ContentDisposition) -> DateTime



Set: ReadDate(self: ContentDisposition)=value

"""

 Size=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the size of a file attachment.



Get: Size(self: ContentDisposition) -> Int64



Set: Size(self: ContentDisposition)=value

"""



class ContentType(object):
 """
 Represents a MIME protocol Content-Type header.

 

 ContentType()

 ContentType(contentType: str)
 """
 def Equals(self,rparam):
  """
  Equals(self: ContentType,rparam: object) -> bool

  

   Determines whether the content-type header of the specified System.Net.Mime.ContentType object 

    is equal to the content-type header of this object.

  

  

   rparam: The System.Net.Mime.ContentType object to compare with this object.

   Returns: true if the content-type headers are the same; otherwise false.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: ContentType) -> int

  

   Determines the hash code of the specified System.Net.Mime.ContentType object

   Returns: An integer hash value.
  """
  pass
 def ToString(self):
  """
  ToString(self: ContentType) -> str

  

   Returns a string representation of this System.Net.Mime.ContentType object.

   Returns: A System.String that contains the current settings for this System.Net.Mime.ContentType.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,contentType=None):
  """
  __new__(cls: type)

  __new__(cls: type,contentType: str)
  """
  pass
 def __ne__(self,*args):
  pass
 Boundary=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the value of the boundary parameter included in the Content-Type header represented by this instance.



Get: Boundary(self: ContentType) -> str



Set: Boundary(self: ContentType)=value

"""

 CharSet=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the value of the charset parameter included in the Content-Type header represented by this instance.



Get: CharSet(self: ContentType) -> str



Set: CharSet(self: ContentType)=value

"""

 MediaType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the media type value included in the Content-Type header represented by this instance.



Get: MediaType(self: ContentType) -> str



Set: MediaType(self: ContentType)=value

"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the value of the name parameter included in the Content-Type header represented by this instance.



Get: Name(self: ContentType) -> str



Set: Name(self: ContentType)=value

"""

 Parameters=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the dictionary that contains the parameters included in the Content-Type header represented by this instance.



Get: Parameters(self: ContentType) -> StringDictionary



"""



class DispositionTypeNames(object):
 """ Supplies the strings used to specify the disposition type for an e-mail attachment. """
 Attachment='attachment'
 Inline='inline'
 __all__=[
  'Attachment',
  'Inline',
 ]


class MediaTypeNames(object):
 """ Specifies the media type information for an e-mail message attachment. """
 Application=None
 Image=None
 Text=None
 __all__=[
  'Application',
  'Image',
  'Text',
 ]


class TransferEncoding(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the Content-Transfer-Encoding header information for an e-mail message attachment.

 

 enum TransferEncoding,values: Base64 (1),EightBit (3),QuotedPrintable (0),SevenBit (2),Unknown (-1)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Base64=None
 EightBit=None
 QuotedPrintable=None
 SevenBit=None
 Unknown=None
 value__=None





class ExecutionEngineException(SystemException,ISerializable,_Exception):
 """
 The exception that is thrown when there is an internal error in the execution engine of the common language runtime. This class cannot be inherited.

 

 ExecutionEngineException()

 ExecutionEngineException(message: str)

 ExecutionEngineException(message: str,innerException: Exception)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,message=None,innerException=None):
  """
  __new__(cls: type)

  __new__(cls: type,message: str)

  __new__(cls: type,message: str,innerException: Exception)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass



class ApplicationIdentity(object,ISerializable):
 """
 Provides the ability to uniquely identify a manifest-activated application. This class cannot be inherited.

 

 ApplicationIdentity(applicationIdentityFullName: str)
 """
 def ToString(self):
  """
  ToString(self: ApplicationIdentity) -> str

  

   Returns the full name of the manifest-activated application.

   Returns: The full name of the manifest-activated application.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,applicationIdentityFullName):
  """ __new__(cls: type,applicationIdentityFullName: str) """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 CodeBase=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the location of the deployment manifest as a URL.



Get: CodeBase(self: ApplicationIdentity) -> str



"""

 FullName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the full name of the application.



Get: FullName(self: ApplicationIdentity) -> str



"""


